@@ -21,49 +21,64 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "linphonecore.h"
 #include "private.h"
 #include "mediastreamer2/mediastream.h"
-#include <eXosip.h>
+#include <eXosip2/eXosip.h>
 #include <osipparser2/osip_message.h>
 
+int linphone_answer_sdp(LinphoneCore *lc, eXosip_event_t *ev, sdp_message_t *sdp);
 
-extern void linphone_register_authentication_required(LinphoneCore *lc,eXosip_event_t *ev);
-extern void linphone_invite_authentication_required(LinphoneCore* lc,LinphoneCall *lcall);
-extern void linphone_subscription_new(LinphoneCore *lc,int did, int sid, char *from, char* contact);
-extern void linphone_notify_recv(LinphoneCore *lc,char *from,int ss_status);
-extern LinphoneProxyConfig *linphone_core_get_proxy_config_from_rid(LinphoneCore *lc, int rid);
-void linphone_subscription_answered(LinphoneCore *lc,char *from, int sid);
-void linphone_subscription_closed(LinphoneCore *lc,char *from, int did);
-
-
-
-
-int linphone_call_accepted(LinphoneCore *lc,int id, const char *sdpbody)
-{
-	StreamParams *audio_params;
-	LinphoneCall *call=lc->call;
-	if (call==NULL){
-		ms_warning("No call to accept.");
-		return 0;
-	}
-	audio_params=&call->audio_params;
-	call->auth_pending=FALSE;
-	if (strlen(sdpbody)==0) {
-		ms_warning("No sdp body in 200ok.");
-		return 0;
-	}
-	if (call->state==LCStateAVRunning){
-		return 0; /*already accepted*/
-	}
-	sdp_context_read_answer(lc->call->sdpctx,sdpbody);
+static void linphone_connect_incoming(LinphoneCore *lc){
 	lc->vtable.show(lc);
 	lc->vtable.display_status(lc,_("Connected."));
-	gstate_new_state(lc, GSTATE_CALL_OUT_CONNECTED, NULL);
 	lc->call->state=LCStateAVRunning;
 	if (lc->ringstream!=NULL){
 		ring_stop(lc->ringstream);
 		lc->ringstream=NULL;
 	}
-	linphone_core_start_media_streams(lc,call);
-	
+	if (lc->audiostream->ticker!=NULL){
+		/*case where we accepted early media */
+		linphone_core_stop_media_streams(lc);
+		linphone_core_init_media_streams(lc);
+	}
+	linphone_core_start_media_streams(lc,lc->call);
+}
+
+int linphone_call_accepted(LinphoneCore *lc, eXosip_event_t *ev)
+{
+	LinphoneCall *call=lc->call;
+	sdp_message_t *sdp=eXosip_get_sdp_info(ev->response);
+	const char *sdpanswer=NULL;
+	osip_message_t *msg=NULL;
+	int err;
+	if (call==NULL){
+		ms_warning("No call to accept.");
+		return 0;
+	}
+	call->auth_pending=FALSE;
+	if (call->state==LCStateAVRunning){
+		return 0; /*already accepted*/
+	}
+	linphone_call_init_media_params(call);
+	if (!lc->sip_conf.sdp_200_ack){
+		err=0;
+		sdp_context_read_answer(call->sdpctx,sdp);
+	}else{
+		/*we receive a 200OK with an sdp offer*/
+		err=linphone_answer_sdp(lc,ev,sdp);
+		if (err==0) sdpanswer=call->sdpctx->answerstr;
+	}
+	if (err==0){
+		gstate_new_state(lc, GSTATE_CALL_OUT_CONNECTED, NULL);
+		linphone_connect_incoming(lc);
+	}
+	/*send the ack once streams are started*/
+	eXosip_call_build_ack(ev->did,&msg);
+	if (sdpanswer!=NULL) linphone_set_sdp(msg,sdpanswer);
+	eXosip_call_send_ack(ev->did,msg);
+	if (err!=0){
+		/*send a bye*/
+		ms_error("Incompatible SDP offer received in 200Ok, need to abort the call");
+		linphone_core_terminate_call(lc,NULL);
+	}
 	return 0;
 }
 
@@ -88,8 +103,12 @@ int linphone_call_terminated(LinphoneCore *lc, eXosip_event_t *ev)
 	lc->vtable.show(lc);
 	lc->vtable.display_status(lc,_("Call terminated."));
 	gstate_new_state(lc, GSTATE_CALL_END, NULL);
-	if (lc->vtable.bye_recv!=NULL)
-		lc->vtable.bye_recv(lc,ev->remote_uri);
+	if (lc->vtable.bye_recv!=NULL){
+		char *from;
+		osip_from_to_str(ev->request->from,&from);
+		lc->vtable.bye_recv(lc,from);
+		osip_free(from);
+	}
 	if (lc->call!=NULL){
 		linphone_call_destroy(lc->call);
 		lc->call=NULL;
@@ -110,9 +129,9 @@ int linphone_call_released(LinphoneCore *lc, int cid){
 	return 0;
 }
 
-int linphone_call_failure(LinphoneCore *lc,int cid, int code)
+int linphone_call_failure(LinphoneCore *lc, eXosip_event_t *ev)
 {
-	const char *reason;
+	const char *reason="";
 	char *msg486=_("User is busy.");
 	char *msg480=_("User is temporarily unavailable.");
 	char *msg487=_("Request Cancelled.");
@@ -120,15 +139,20 @@ int linphone_call_failure(LinphoneCore *lc,int cid, int code)
 	char *msg600=_("User does not want to be disturbed.");
 	char *msg603=_("Call declined.");
 	char* tmpmsg=msg486;
+	int code;
 
-	
+	if (ev->response){
+		code=osip_message_get_status_code(ev->response);
+		reason=osip_message_get_reason_phrase(ev->response);
+	}else code=-110;
 	lc->vtable.show(lc);
 	
 	switch(code)
 	{
 		case 401:
 		case 407:
-			if (lc->call!=NULL) linphone_invite_authentication_required(lc,lc->call);
+			if (lc->call!=NULL)
+				linphone_process_authentication(lc,ev);
 			return 0;
 			break;
 		case 400:
@@ -173,8 +197,7 @@ int linphone_call_failure(LinphoneCore *lc,int cid, int code)
 		default:
 			if (code>0)
 			{
-				reason=osip_message_get_reason(code);
-				lc->vtable.display_status(lc,(char*)reason);
+				lc->vtable.display_status(lc,reason);
 			}
 			else ms_warning("failure_cb unknown code=%i\n",code);	
 	}
@@ -193,164 +216,162 @@ int linphone_call_failure(LinphoneCore *lc,int cid, int code)
 
 extern sdp_handler_t linphone_sdphandler;
 
-int linphone_inc_new_call(LinphoneCore *lc, eXosip_event_t *ev)
-{
-	char *barmesg,*answer;
+int linphone_answer_sdp(LinphoneCore *lc, eXosip_event_t *ev, sdp_message_t *sdp){
 	int status=200;
 	sdp_context_t *ctx=NULL;
-	osip_from_t *from_url=NULL;
-	int cid=ev->cid;
-	int did=ev->did;
-	char *from=ev->remote_uri;
-	char *sdp_body=ev->sdp_body;
-#ifdef VINCENT_MAURY_RSVP
-	char *textinfo=ev->textinfo;
- #endif
 	
-	osip_from_init(&from_url);
-	osip_from_parse(from_url,from);
+	ctx=lc->call->sdpctx;
+	/* get the result of the negociation */
+	sdp_context_get_answer(ctx,sdp);
+	status=sdp_context_get_status(ctx);
+
+	if (status==200){
+		linphone_core_init_media_streams(lc);
+		return 0;
+	}else{
+		if (status==-1) status=415;
+	}
+	return -1;
+}
+
+int linphone_inc_new_call(LinphoneCore *lc, eXosip_event_t *ev)
+{
+	sdp_message_t *sdp=eXosip_get_sdp_info(ev->request);
+	osip_from_t *from_url=ev->request->from;
+	char *barmesg;
+	char *from;
+	char *to;
+	int err;
+
+	osip_from_to_str(ev->request->from,&from);
+	osip_to_to_str(ev->request->to,&to);	
 	
 	/* first check if we can answer successfully to this invite */
 	if (lc->presence_mode!=LINPHONE_STATUS_ONLINE){
 		ms_message("Not present !! presence mode : %d\n",lc->presence_mode);
 		eXosip_lock();
 		if (lc->presence_mode==LINPHONE_STATUS_BUSY)
-		  
-		  eXosip_answer_call(did,486,NULL);
+			eXosip_call_send_answer(ev->tid,486,NULL);
 		else if (lc->presence_mode==LINPHONE_STATUS_AWAY
 			 ||lc->presence_mode==LINPHONE_STATUS_BERIGHTBACK
 			 ||lc->presence_mode==LINPHONE_STATUS_ONTHEPHONE
 			 ||lc->presence_mode==LINPHONE_STATUS_OUTTOLUNCH
 			 ||lc->presence_mode==LINPHONE_STATUS_OFFLINE)
-		  eXosip_answer_call(did,480,NULL);
+		  eXosip_call_send_answer(ev->tid,480,NULL);
 		else if (lc->presence_mode==LINPHONE_STATUS_NOT_DISTURB)
-		  eXosip_answer_call(did,480,NULL);
+		  eXosip_call_send_answer(ev->tid,480,NULL);
 		else if (lc->alt_contact!=NULL && lc->presence_mode==LINPHONE_STATUS_MOVED)
 		  {
-		    eXosip_set_redirection_address (did, lc->alt_contact);
-		    eXosip_answer_call(did,302,NULL);
-		    eXosip_set_redirection_address (did, NULL);
+			osip_message_t *msg;
+			eXosip_call_build_answer(ev->tid,302,&msg);
+			osip_message_set_contact(msg,lc->alt_contact);
+			eXosip_call_send_answer(ev->tid,302,msg);
 		  }
 		else if (lc->alt_contact!=NULL && lc->presence_mode==LINPHONE_STATUS_ALT_SERVICE)
 		  {
-		    eXosip_set_redirection_address (did, lc->alt_contact);
-		    eXosip_answer_call(did,380,NULL);
-		    eXosip_set_redirection_address (did, NULL);
+			osip_message_t *msg;
+			eXosip_call_build_answer(ev->tid,380,&msg);
+			osip_message_set_contact(msg,lc->alt_contact);
+			eXosip_call_send_answer(ev->tid,380,msg);
 		  }
 		else
-		  eXosip_answer_call(did,486,NULL);
+		  eXosip_call_send_answer(ev->tid,486,NULL);
 		eXosip_unlock();
 		goto end;
 	}
-	/* not busy */
-#ifdef VINCENT_MAURY_RSVP
-	/* in qos mode, call already exists at the end of resource reservation */
-	if (lc->call!=NULL && lc->rsvp_enable && strcmp(textinfo,"New Call")!=0){
-#else
-	if (lc->call!=NULL){
-#endif
+	if (lc->call!=NULL){/*busy*/
 		eXosip_lock();
-		eXosip_answer_call(did,486,NULL);
+		eXosip_call_send_answer(ev->tid,486,NULL);
 		eXosip_unlock();
 		goto end;
 	}
-	if (strlen(sdp_body)==0){
-		ms_warning("No sdp body !");
-		eXosip_lock();
-		eXosip_answer_call(did,603,NULL);
-		eXosip_unlock();
-		goto end;
+	lc->call=linphone_call_new_incoming(lc,from,to,ev->cid,ev->did,ev->tid);
+	gstate_new_state(lc, GSTATE_CALL_IN_INVITE, NULL);
+	if (sdp==NULL){
+		ms_message("No sdp body in invite, 200-ack scheme");
+		err=0;
+	}else{
+		err=linphone_answer_sdp(lc,ev,sdp);
 	}
-#ifdef VINCENT_MAURY_RSVP
-	/* make the negotiation only once at the first invite received */
-	if (strcmp(textinfo,"New Call")!=0)
-	{
+	if (!err){
+		char *tmp;
+		if (from_2char_without_params(from_url,&tmp)!=0){
+			tmp=ms_strdup("Unknown user");
+		}
+		barmesg=ortp_strdup_printf("%s %s",tmp,_("is contacting you."));
+		lc->vtable.show(lc);
+		lc->vtable.display_status(lc,barmesg);
+		lc->vtable.inv_recv(lc,tmp);
+		ms_free(barmesg);
+		osip_free(tmp);
 		
-		lc->call=linphone_call_new_incoming(lc,from,ev->local_uri,cid,did);
-		ctx=lc->call->sdpctx;
-		/* get the result of the negociation */
-		answer=sdp_context_get_answer(ctx,sdp_body);
-		status=sdp_context_get_status(ctx);
-	}
-#else
-	lc->call=linphone_call_new_incoming(lc,from,ev->local_uri,cid,did);
-	ctx=lc->call->sdpctx;
-	/* get the result of the negociation */
-	answer=sdp_context_get_answer(ctx,sdp_body);
-	status=sdp_context_get_status(ctx);
-
-#endif
-	switch(status)
-	{
-		case 200:
-			/* vérification de la qos */
-#ifdef VINCENT_MAURY_RSVP
-			if (strcmp(textinfo,"With QoS")==0 && !lc->rsvp_enable)
-				/* caller has QoS, we don't */
-				lc->vtable.display_yes_no(lc,strdup(textinfo));
-			else if (strcmp(textinfo,"Without QoS")==0 && lc->rsvp_enable)
-				/* caller doesn't have QoS, we do ! */
-				lc->vtable.display_yes_no(lc,strdup(textinfo));
-			else if (strcmp(textinfo,"With QoS")==0 && lc->rsvp_enable)
-			{
-				/* we agree on With QoS */
-				eXosip_lock();
-				eXosip_answer_call_with_body(did,183,"application/sdp",answer);
-				eXosip_unlock();
-			}
-			else if (strcmp(textinfo,"Without QoS")==0 && !lc->rsvp_enable ||
-					strcmp(textinfo,"New Call")==0)
-			{
-				/* we agree on Without QoS */
-				eXosip_lock();
-				eXosip_answer_call(did,180,NULL);
-				eXosip_unlock();
-				/* play the ring */
-				lc->ringstream=ring_start(lc->sound_conf.local_ring,
-							2000,lc->sound_conf.ring_sndcard);
-			}
-#else
-			eXosip_lock();
-			eXosip_answer_call(did,180,NULL);
-			eXosip_unlock();
-			/* play the ring */
-			if (lc->sound_conf.ring_sndcard!=NULL){
-				ms_message("Starting local ring...");
-				lc->ringstream=ring_start(lc->sound_conf.local_ring,2000,lc->sound_conf.ring_sndcard);
-			}
-#endif
-			linphone_call_set_state(lc->call,LCStateRinging);
-			{
-				char *tmp;
-				if (from_2char_without_params(from_url,&tmp)!=0){
-					tmp=ms_strdup("Unknown user");
-				}
-				linphone_core_init_media_streams(lc);
-				barmesg=ortp_strdup_printf("%s %s",tmp,_("is contacting you."));
-				lc->vtable.show(lc);
-				lc->vtable.display_status(lc,barmesg);
-				gstate_new_state(lc, GSTATE_CALL_IN_INVITE, tmp);
-				lc->vtable.inv_recv(lc,tmp);
-				ms_free(barmesg);
-				osip_free(tmp);
-				
-				gstate_new_state(lc, GSTATE_CALL_IN_INVITE, NULL);
-			}
-		break;
-		default:	
-			if (status==-1) status=415;
-			ms_warning("Error during sdp negociation. status : %d\n",status);
-			eXosip_lock();
-			eXosip_answer_call(did,status,NULL);
-			eXosip_unlock();
-			linphone_call_destroy(lc->call);
-			lc->call=NULL;
+		linphone_call_set_state(lc->call,LCStateRinging);
+		eXosip_lock();
+		eXosip_call_send_answer(ev->tid,180,NULL);
+		eXosip_unlock();
+		/* play the ring */
+		if (lc->sound_conf.ring_sndcard!=NULL){
+			ms_message("Starting local ring...");
+			lc->ringstream=ring_start(lc->sound_conf.local_ring,2000,lc->sound_conf.ring_sndcard);
+		}
+	}else{
+		ms_error("Error during sdp negociation. ");
+		eXosip_lock();
+		eXosip_call_send_answer(ev->tid,415,NULL);
+		eXosip_unlock();
+		linphone_call_destroy(lc->call);
+		lc->call=NULL;
 	}
 	end:
-		osip_from_free(from_url);
+	osip_free(from);
+	osip_free(to);
 	return 0;
 }
 
+void linphone_handle_ack(LinphoneCore *lc, eXosip_event_t *ev){
+	sdp_message_t *sdp=eXosip_get_sdp_info(ev->ack);
+	if (sdp){
+		sdp_context_read_answer(lc->call->sdpctx,sdp);
+		linphone_connect_incoming(lc);
+	}
+}
+
+void linphone_handle_reinvite(LinphoneCore *lc, eXosip_event_t *ev){
+	sdp_message_t *sdp=eXosip_get_sdp_info(ev->request);
+	sdp_context_t *ctx;
+	LinphoneCall *call=lc->call;
+	char *answer;
+	int status;
+	if (sdp==NULL){
+		ms_warning("No sdp in reinvite !");
+		eXosip_lock();
+		eXosip_call_send_answer(ev->tid,603,NULL);
+		eXosip_unlock();
+		return;
+	}
+	ctx=call->sdpctx;
+	/* get the result of the negociation */
+	linphone_call_init_media_params(call);
+	answer=sdp_context_get_answer(ctx,sdp);
+	status=sdp_context_get_status(ctx);
+	if (status==200){
+		osip_message_t *msg=NULL;
+		linphone_core_stop_media_streams(lc);
+		linphone_core_init_media_streams(lc);
+		eXosip_lock();
+		eXosip_call_build_answer(ev->tid,200,&msg);
+		answer=call->sdpctx->answerstr;	/* takes the sdp already computed*/
+		linphone_set_sdp(msg,answer);
+		eXosip_call_send_answer(ev->tid,200,msg);
+		eXosip_unlock();
+		linphone_core_start_media_streams(lc,call);
+	}else{
+		eXosip_lock();
+		eXosip_call_send_answer(ev->tid,status,NULL);
+		eXosip_unlock();
+	}
+}
+
 void linphone_do_automatic_redirect(LinphoneCore *lc, const char *contact){
 	char *msg=ortp_strdup_printf(_("Redirected to %s..."),contact);
 	lc->vtable.display_status(lc,msg);
@@ -360,10 +381,15 @@ void linphone_do_automatic_redirect(LinphoneCore *lc, const char *contact){
 	linphone_core_invite(lc,contact);
 }
 
-void linphone_call_redirected(LinphoneCore *lc, int cid,int code, const char *contact){
+void linphone_call_redirected(LinphoneCore *lc, eXosip_event_t *ev){
+	int code=osip_message_get_status_code(ev->response);
+	char *contact=NULL;
+	osip_contact_t *ct;
+	osip_message_get_contact(ev->response,0,&ct);
+	if (ct) osip_contact_to_str(ct,&contact);
 	switch(code){
 		case 380:
-			lc->vtable.display_url(lc,_("User is not reachable at the moment but he invites you\nto contact him using the following alternate resource:"),(char*)contact);
+			lc->vtable.display_url(lc,_("User is not reachable at the moment but he invites you\nto contact him using the following alternate resource:"),contact);
 			if (lc->call!=NULL) linphone_call_destroy(lc->call);
 			lc->call=NULL;
 			break;
@@ -371,6 +397,7 @@ void linphone_call_redirected(LinphoneCore *lc, int cid,int code, const char *co
 			linphone_do_automatic_redirect(lc,contact);
 			break;
 	}
+	if (contact) osip_free(contact);
 }
 
 
@@ -510,6 +537,12 @@ SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payl
 				payload_type_set_send_fmtp(rtppayload,payload->a_fmtp);
 			}
 			payload->a_fmtp=rtppayload->recv_fmtp;
+			if (payload->a_ptime>0){
+				char tmp[30];
+				snprintf(tmp,sizeof(tmp),"ptime=%i",payload->a_ptime);
+				payload_type_append_send_fmtp(rtppayload,tmp);
+				ms_message("%s attribute added to fmtp");
+			}
 		}
 		return ret;
 	}
@@ -602,7 +635,7 @@ int linphone_read_audio_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 	/* paranoid check: see if this codec is supported in our local rtp profile*/
 	supported=linphone_payload_is_supported(lc, payload,lc->local_profile,call->profile,FALSE);
 	if (supported==Unsupported) {
-		ms_warning("This remote sip phone did not answer properly to my sdp offer!");
+		ms_warning("This remote sip phone did not answer properly to my sdp offer: rtpmap=%s",payload->a_rtpmap);
 		return 0;
 	}
 	if (supported==SupportedAndValid){
@@ -630,7 +663,7 @@ int linphone_read_video_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 	/* paranoid check: see if this codec is supported in our local rtp profile*/
 	supported=linphone_payload_is_supported(lc, payload,lc->local_profile,call->profile,FALSE);
 	if (supported==Unsupported) {
-		ms_warning("This remote sip phone did not answer properly to my sdp offer!");
+		ms_warning("This remote sip phone did not answer properly to my sdp offer: rtpmap=%s",payload->a_rtpmap);
 		return 0;
 	}
 	if (supported==SupportedAndValid){
@@ -648,57 +681,121 @@ int linphone_read_video_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 	return 0;
 }
 
-void linphone_call_proceeding(LinphoneCore *lc, int cid, int did){
+void linphone_call_proceeding(LinphoneCore *lc, eXosip_event_t *ev){
 	if (lc->call==NULL){
 		ms_warning("Bug in call_proceeding()");
 		return;
 	}
-	lc->call->cid=cid;
-	lc->call->did=did;
-
+	lc->call->cid=ev->cid;
+	lc->call->did=ev->did;
+	lc->call->tid=ev->tid;
 }
 
 void linphone_call_ringing(LinphoneCore *lc, eXosip_event_t *ev){
+	sdp_message_t *sdp=eXosip_get_sdp_info(ev->response);
+	LinphoneCall *call=lc->call;
 	
-	if (lc->audiostream->ticker!=NULL) return; /*already listening to remote host */
-	linphone_call_proceeding(lc,ev->cid,ev->did);
+	linphone_call_proceeding(lc,ev);
 	
-	if (ev->status_code==180){
+
+	if (sdp==NULL){
 		if (lc->ringstream!=NULL) return;	/*already ringing !*/
 		if (lc->sound_conf.play_sndcard!=NULL){
 			ms_message("Remote ringing...");
 			lc->ringstream=ring_start(lc->sound_conf.remote_ring,2000,lc->sound_conf.play_sndcard);
 		}
-	}else if (strlen(ev->sdp_body)>0){
-		/* somebody reported a server who uses a 183 answer to establish mediastreams and play
-		an informationnal message */
-		linphone_call_accepted(lc,ev->did,ev->sdp_body);
+	}else{
+		/*accept early media */
+		StreamParams *audio_params;
+		if (call==NULL){
+			ms_error("No call ?");
+			goto end;
+		}
+		if (lc->audiostream->ticker!=NULL){
+			/*streams already started */
+			ms_message("Early media already started.");
+			goto end;
+		}
+		audio_params=&call->audio_params;
+		sdp_context_read_answer(lc->call->sdpctx,sdp);
+		lc->vtable.show(lc);
+		lc->vtable.display_status(lc,_("Early media."));
+		gstate_new_state(lc, GSTATE_CALL_OUT_CONNECTED, NULL);
+		if (lc->ringstream!=NULL){
+			ring_stop(lc->ringstream);
+			lc->ringstream=NULL;
+		}
+		ms_message("Doing early media...");
+		linphone_core_start_media_streams(lc,call);
 	}
-	
+	call->state=LCStateRinging;
+	goto end;
+	end:
+		sdp_message_free(sdp);
+
+}
+
+void linphone_call_message_new(LinphoneCore *lc, eXosip_event_t *ev){
+#ifdef VIDEO_ENABLED
+	if (ev->request){
+		if (MSG_IS_INFO(ev->request)){
+			osip_content_type_t *ct;
+			ct=osip_message_get_content_type(ev->request);
+			if (ct && ct->subtype &&
+				strcmp(ct->subtype,"media_control+xml")==0){
+				osip_body_t *body=NULL;
+				osip_message_get_body(ev->request,0,&body);
+				if (body && body->body!=NULL &&
+					strstr(body->body,"picture_fast_update")){
+					osip_message_t *ans=NULL;
+					ms_message("Receiving VFU request !");
+					if (lc->videostream)
+						video_stream_send_vfu(lc->videostream);
+					eXosip_call_build_answer(ev->tid,200,&ans);
+					if (ans)
+						eXosip_call_send_answer(ev->tid,200,ans);
+				}
+			}	
+		}
+	}else ms_warning("linphone_call_message_new: No request ?");
+#endif
 }
 
 
-void linphone_registration_faillure(LinphoneCore *lc,eXosip_event_t *ev)
-{
+void linphone_registration_faillure(LinphoneCore *lc, eXosip_event_t *ev){
+	int status_code=0;
 	char *msg;
-	switch(ev->status_code){
+	const char *reason=NULL;
+	osip_uri_t *requri=osip_message_get_uri(ev->request);
+	char *ru;
+	
+	osip_uri_to_str(requri,&ru);
+
+	if (ev->response){
+		status_code=osip_message_get_status_code(ev->response);
+		reason=osip_message_get_reason_phrase(ev->response);
+	}
+	switch(status_code){
 		case 401:
 		case 407:
-			linphone_register_authentication_required(lc,ev);
+			linphone_process_authentication(lc,ev);
 			break;
 		default:
-			if (strlen(ev->reason_phrase)==0) msg=ortp_strdup_printf(_("Registration on %s failed (timeout)."),ev->req_uri);
-			else msg=ev->reason_phrase;
+			msg=ortp_strdup_printf(_("Registration on %s failed: %s"),ru,(reason!=NULL) ? reason : _("no response timeout"));
 			lc->vtable.display_status(lc,msg);
 			gstate_new_state(lc, GSTATE_REG_FAILED, msg);
-			if ( msg != ev->reason_phrase ) ms_free(msg);
+			ms_free(msg);
 	}
-	
+	osip_free(ru);
 }
 
 void linphone_registration_success(LinphoneCore *lc,eXosip_event_t *ev){
 	LinphoneProxyConfig *cfg;
-	char *msg=ms_strdup_printf(_("Registration on %s successful."),ev->req_uri);
+	osip_uri_t *requri=osip_message_get_uri(ev->request);
+	char *msg;
+	char *ru;
+	osip_uri_to_str(requri,&ru);
+	msg=ms_strdup_printf(_("Registration on %s successful."),ru);
 	lc->vtable.display_status(lc,msg);
 	ms_free(msg);
 	cfg=linphone_core_get_proxy_config_from_rid(lc,ev->rid);
@@ -713,9 +810,8 @@ void linphone_core_process_event(LinphoneCore *lc,eXosip_event_t *ev)
 	switch(ev->type){
 		case EXOSIP_CALL_ANSWERED:
 			ms_message("CALL_ANSWERED\n");
-			linphone_call_accepted(lc,ev->did,ev->sdp_body);
-			/*send the ack once streams are started*/
-			eXosip_send_ack(ev->did);
+			linphone_call_accepted(lc,ev);
+			linphone_authentication_ok(lc,ev);
 			break;
 		case EXOSIP_CALL_CLOSED:
 		case EXOSIP_CALL_CANCELLED:
@@ -725,56 +821,61 @@ void linphone_core_process_event(LinphoneCore *lc,eXosip_event_t *ev)
 		case EXOSIP_CALL_TIMEOUT:
 		case EXOSIP_CALL_NOANSWER:
 			ms_message("CALL_TIMEOUT or NOANSWER\n");
-			linphone_call_failure(lc,ev->did,-110);
+			linphone_call_failure(lc,ev);
 			break;
 		case EXOSIP_CALL_REQUESTFAILURE:
 		case EXOSIP_CALL_GLOBALFAILURE:
 		case EXOSIP_CALL_SERVERFAILURE:
 			ms_message("CALL_REQUESTFAILURE or GLOBALFAILURE or SERVERFAILURE\n");
-			linphone_call_failure(lc,ev->did,ev->status_code);
+			linphone_call_failure(lc,ev);
 			break;
-		case EXOSIP_CALL_NEW:
+		case EXOSIP_CALL_INVITE:
 			ms_message("CALL_NEW\n");
 			/* CALL_NEW is used twice in qos mode : 
 			 * when you receive invite (textinfo = "With QoS" or "Without QoS")
 			 * and when you receive update (textinfo = "New Call") */
 			linphone_inc_new_call(lc,ev);
 			break;
+		case EXOSIP_CALL_REINVITE:
+			linphone_handle_reinvite(lc,ev);
+			break;
 		case EXOSIP_CALL_ACK:
-			ms_message("CALL_ACK\n");
+			ms_message("CALL_ACK");
+			linphone_handle_ack(lc,ev);
 			break;
 		case EXOSIP_CALL_REDIRECTED:
-			ms_message("CALL_REDIRECTED\n");
-			linphone_call_redirected(lc,ev->did,ev->status_code,ev->remote_contact);
+			ms_message("CALL_REDIRECTED");
+			linphone_call_redirected(lc,ev);
 			break;
 		case EXOSIP_CALL_PROCEEDING:
-			ms_message("CALL_PROCEEDING\n");
-			linphone_call_proceeding(lc,ev->cid,ev->did);
+			ms_message("CALL_PROCEEDING");
+			linphone_call_proceeding(lc,ev);
 			break;
 		case EXOSIP_CALL_RINGING:
-			ms_message("CALL_RINGING\n");
+			ms_message("CALL_RINGING");
 			linphone_call_ringing(lc,ev);
 			break;
+		case EXOSIP_CALL_MESSAGE_NEW:
+			ms_message("EXOSIP_CALL_MESSAGE_NEW");
+			linphone_call_message_new(lc,ev);
+			break;
 		case EXOSIP_IN_SUBSCRIPTION_NEW:
-			ms_message("CALL_SUBSCRIPTION_NEW or UPDATE\n");
-			linphone_subscription_new(lc,ev->did,ev->nid,ev->remote_uri,ev->remote_contact);
+			ms_message("CALL_SUBSCRIPTION_NEW or UPDATE");
+			linphone_subscription_new(lc,ev);
+			break;
+		case EXOSIP_SUBSCRIPTION_UPDATE:
 			break;
 		case EXOSIP_SUBSCRIPTION_NOTIFY:
-			ms_message("CALL_SUBSCRIPTION_NOTIFY\n");
-			linphone_notify_recv(lc,ev->remote_uri,ev->online_status);
+			ms_message("CALL_SUBSCRIPTION_NOTIFY");
+			linphone_notify_recv(lc,ev);
 			break;
 		case EXOSIP_SUBSCRIPTION_ANSWERED:
-			linphone_subscription_answered(lc,ev->remote_uri, ev->did);
+			ms_message("EXOSIP_SUBSCRIPTION_ANSWERED, ev->sid=%i\n",ev->sid);
+			linphone_subscription_answered(lc,ev);
 			break;
 		case EXOSIP_SUBSCRIPTION_CLOSED:
-			linphone_subscription_closed(lc,ev->remote_uri,ev->did);
-			break;
-		case EXOSIP_CALL_HOLD:
-		case EXOSIP_CALL_OFFHOLD:
-			ms_message("CALL_HOLD ou OFFHOLD\n");
-			break;
-		case EXOSIP_CALL_STARTAUDIO:
-			ms_message("CALL_STARTAUDIO\n");
+			ms_message("EXOSIP_SUBSCRIPTION_CLOSED\n");
+			linphone_subscription_closed(lc,ev);
 			break;
 		case EXOSIP_CALL_RELEASED:
 			ms_message("CALL_RELEASED\n");
@@ -786,9 +887,11 @@ void linphone_core_process_event(LinphoneCore *lc,eXosip_event_t *ev)
 			break;
 		case EXOSIP_REGISTRATION_SUCCESS:
 			linphone_registration_success(lc,ev);
+			linphone_authentication_ok(lc,ev);
 			break;
 		case EXOSIP_MESSAGE_NEW:
-			linphone_core_text_received(lc,ev->remote_uri,ev->sdp_body);
+			linphone_core_text_received(lc,ev);
+			break;
 		default:
 			break;
 	}
