@@ -21,7 +21,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "lpconfig.h"
 #include "private.h"
 #include "mediastreamer2/mediastream.h"
-#include <eXosip.h>
+#include <eXosip2/eXosip.h>
 #include "sdphandler.h"
 
 #include <ortp/telephonyevents.h>
@@ -35,6 +35,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 #endif
 
+#ifdef WIN32
+#define HAVE_EXOSIP_GET_VERSION 1
+#endif
+
 
 static const char *liblinphone_version=LIBLINPHONE_VERSION;
 
@@ -78,6 +82,12 @@ static void  linphone_call_init_common(LinphoneCall *call, char *from, char *to)
 	call->state=LCStateInit;
 	call->start_time=time(NULL);
 	call->log=linphone_call_log_new(call, from, to);
+	linphone_core_notify_all_friends(call->core,LINPHONE_STATUS_ONTHEPHONE);
+}
+
+void linphone_call_init_media_params(LinphoneCall *call){
+	memset(&call->audio_params,0,sizeof(call->audio_params));
+	memset(&call->video_params,0,sizeof(call->video_params));
 }
 
 LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_from_t *from, const osip_to_t *to)
@@ -88,10 +98,10 @@ LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_f
 	call->dir=LinphoneCallOutgoing;
 	call->cid=-1;
 	call->did=-1;
+	call->tid=-1;
 	call->core=lc;
 	linphone_core_get_local_ip(lc,to->url->host,localip);
 	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,localip,from->url->username);
-	call->profile=lc->local_profile; /*use the readonly local profile*/
 	osip_from_to_str(from,&fromstr);
 	osip_to_to_str(to,&tostr);
 	linphone_call_init_common(call,fromstr,tostr);
@@ -99,7 +109,7 @@ LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_f
 }
 
 
-LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, const char *to, int cid, int did)
+LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, const char *to, int cid, int did, int tid)
 {
 	char localip[LINPHONE_IPADDR_SIZE];
 	LinphoneCall *call=ms_new0(LinphoneCall,1);
@@ -108,6 +118,7 @@ LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, co
 	call->dir=LinphoneCallIncoming;
 	call->cid=cid;
 	call->did=did;
+	call->tid=tid;
 	call->core=lc;
 	osip_from_init(&from_url);
 	osip_from_parse(from_url, from);
@@ -121,6 +132,7 @@ LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, co
 
 void linphone_call_destroy(LinphoneCall *obj)
 {
+	linphone_core_notify_all_friends(obj->core,obj->core->prev_mode);
 	linphone_call_log_completed(obj->log,obj);
 	if (obj->profile!=NULL && obj->profile!=obj->core->local_profile) rtp_profile_destroy(obj->profile);
 	if (obj->sdpctx!=NULL) sdp_context_free(obj->sdpctx);
@@ -211,12 +223,55 @@ void linphone_call_log_destroy(LinphoneCallLog *cl){
 	ms_free(cl);
 }
 
+void _osip_trace_func(char *fi, int li, osip_trace_level_t level, char *chfr, va_list ap){
+	int ortp_level=ORTP_DEBUG;
+	switch(level){
+		case OSIP_INFO1:
+		case OSIP_INFO2:
+		case OSIP_INFO3:
+		case OSIP_INFO4:
+			ortp_level=ORTP_MESSAGE;
+			break;
+		case OSIP_WARNING:
+			ortp_level=ORTP_WARNING;
+			break;
+		case OSIP_ERROR:
+		case OSIP_BUG:
+			ortp_level=ORTP_ERROR;
+			break;
+		case OSIP_FATAL:
+			ortp_level=ORTP_FATAL;
+			break;
+		case END_TRACE_LEVEL:
+			break;	
+	}
+	if (ortp_log_level_enabled(level)){
+		int len=strlen(chfr);
+		char *chfrdup=ortp_strdup(chfr);
+		/*need to remove endline*/
+		if (len>1){
+			if (chfrdup[len-1]=='\n')
+				chfrdup[len-1]='\0';
+			if (chfrdup[len-2]=='\r')
+				chfrdup[len-2]='\0';
+		}
+		ortp_logv(ortp_level,chfrdup,ap);
+		ortp_free(chfrdup);
+	}
+}
+
 
 void linphone_core_enable_logs(FILE *file){
 	if (file==NULL) file=stdout;
 	ortp_set_log_file(file);
 	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
-	osip_trace_initialize (OSIP_INFO4, file);
+	osip_trace_initialize_func (OSIP_INFO4,&_osip_trace_func);
+}
+
+void linphone_core_enable_logs_with_cb(OrtpLogFunc logfunc){
+	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+	osip_trace_initialize_func (OSIP_INFO4,&_osip_trace_func);
+	ortp_set_log_handler(logfunc);
 }
 
 void linphone_core_disable_logs(){
@@ -244,6 +299,7 @@ net_config_read (LinphoneCore *lc)
 	tmp=lp_config_get_int(lc->config,"net","firewall_policy",0);
 	linphone_core_set_firewall_policy(lc,tmp);
 	tmp=lp_config_get_int(lc->config,"net","nat_sdp_only",0);
+	lc->net_conf.nat_sdp_only=tmp;
 }
 
 
@@ -287,7 +343,7 @@ void sound_config_read(LinphoneCore *lc)
 	
 	tmpbuf=PACKAGE_SOUND_DIR "/" LOCAL_RING;
 	tmpbuf=lp_config_get_string(lc->config,"sound","local_ring",tmpbuf);
-	if (access(tmpbuf,F_OK)==0) {
+	if (access(tmpbuf,F_OK)==-1) {
 		tmpbuf=PACKAGE_SOUND_DIR "/" LOCAL_RING;
 	}
 	if (strstr(tmpbuf,".wav")==NULL){
@@ -299,7 +355,7 @@ void sound_config_read(LinphoneCore *lc)
 	
 	tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
 	tmpbuf=lp_config_get_string(lc->config,"sound","remote_ring",tmpbuf);
-	if (access(tmpbuf,F_OK)==0){
+	if (access(tmpbuf,F_OK)==-1){
 		tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
 	}
 	if (strstr(tmpbuf,".wav")==NULL){
@@ -336,7 +392,7 @@ void sip_config_read(LinphoneCore *lc)
 	linphone_core_set_sip_port(lc,port);
 	
 	tmpstr=lp_config_get_string(lc->config,"sip","contact",NULL);
-	if (tmpstr==NULL) {
+	if (tmpstr==NULL || linphone_core_set_primary_contact(lc,tmpstr)==-1) {
 		char *hostname=getenv("HOST");
 		char *username=getenv("USER");
 		if (hostname==NULL) hostname=getenv("HOSTNAME");
@@ -346,10 +402,9 @@ void sip_config_read(LinphoneCore *lc)
 			username="toto";
 		}
 		contact=ortp_strdup_printf("sip:%s@%s",username,hostname);
-	}else contact=ms_strdup(tmpstr);
-	linphone_core_set_primary_contact(lc,contact);
-	ms_free(contact);
-	
+		linphone_core_set_primary_contact(lc,contact);
+		ms_free(contact);
+	}
 
 	tmp=lp_config_get_int(lc->config,"sip","guess_hostname",1);
 	linphone_core_set_guess_hostname(lc,tmp);
@@ -380,6 +435,8 @@ void sip_config_read(LinphoneCore *lc)
 			break;
 		}
 	}
+	/*for test*/
+	lc->sip_conf.sdp_200_ack=lp_config_get_int(lc->config,"sip","sdp_200_ack",0);
 	
 }
 
@@ -445,15 +502,18 @@ void codecs_config_read(LinphoneCore *lc)
 
 void video_config_read(LinphoneCore *lc)
 {
-	int tmp;
+	int capture, display;
+	int enabled;
 	const char *str;
 	
 	str=lp_config_get_string(lc->config,"video","device","/dev/video0");
 	linphone_core_set_video_device(lc,NULL,str);
 	
-	tmp=lp_config_get_int(lc->config,"video","enabled",1);
+	enabled=lp_config_get_int(lc->config,"video","enabled",1);
+	capture=lp_config_get_int(lc->config,"video","capture",enabled);
+	display=lp_config_get_int(lc->config,"video","display",enabled);
 #ifdef VIDEO_ENABLED
-	linphone_core_enable_video(lc,tmp);
+	linphone_core_enable_video(lc,capture,display);
 #endif
 }
 
@@ -528,8 +588,9 @@ const char * linphone_core_get_version(void){
 	return liblinphone_version;
 }
 
-void
-linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const char *config_path, void * userdata)
+extern PayloadType payload_type_x_snow;
+
+void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const char *config_path, void * userdata)
 {
 	memset (lc, 0, sizeof (LinphoneCore));
 	lc->data=userdata;
@@ -547,14 +608,16 @@ linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const c
 	rtp_profile_set_payload(&av_profile,116,&payload_type_truespeech);
 	rtp_profile_set_payload(&av_profile,101,&payload_type_telephone_event);
 	
+#ifdef VIDEO_ENABLED
 	rtp_profile_set_payload(&av_profile,97,&payload_type_theora);
 	rtp_profile_set_payload(&av_profile,98,&payload_type_h263_1998);
 	rtp_profile_set_payload(&av_profile,99,&payload_type_mp4v);
+	rtp_profile_set_payload(&av_profile,100,&payload_type_x_snow);
+#endif
 
 	ms_init();
 	
 	lc->config=lp_config_new(config_path);
-
   
 #ifdef VINCENT_MAURY_RSVP
 	/* default qos parameters : rsvp on, rpc off */
@@ -568,7 +631,7 @@ linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const c
 	sip_config_read(lc); /* this will start eXosip*/
 	video_config_read(lc);
 	//autoreplier_config_init(&lc->autoreplier_conf);
-	
+	lc->prev_mode=LINPHONE_STATUS_ONLINE;
 	lc->presence_mode=LINPHONE_STATUS_ONLINE;
 	lc->max_call_logs=15;
 	ui_config_read(lc);
@@ -595,18 +658,26 @@ const MSList *linphone_core_get_video_codecs(const LinphoneCore *lc)
 	return lc->codecs_conf.video_codecs;
 }
 
-int linphone_core_set_primary_contact(LinphoneCore *lc,const char *contact)
+int linphone_core_set_primary_contact(LinphoneCore *lc, const char *contact)
 {
+	osip_from_t *ctt=NULL;
+	osip_from_init(&ctt);
+	if (osip_from_parse(ctt,contact)!=0){
+		ms_error("Bad contact url: %s",contact);
+		osip_from_free(ctt);
+		return -1;
+	}
 	if (lc->sip_conf.contact!=NULL) ms_free(lc->sip_conf.contact);
 	lc->sip_conf.contact=ms_strdup(contact);
 	if (lc->sip_conf.guessed_contact!=NULL){
 		ms_free(lc->sip_conf.guessed_contact);
 		lc->sip_conf.guessed_contact=NULL;
 	}
+	osip_from_free(ctt);
 	return 0;
 }
 
-static bool_t stun_get_localip(LinphoneCore *lc, char *result){
+static bool_t stun_get_localip(LinphoneCore *lc, char *result, int *port){
 	const char *server=linphone_core_get_stun_server(lc);
 	StunAddress4 addr;
 	StunAddress4 mapped;
@@ -620,10 +691,11 @@ static bool_t stun_get_localip(LinphoneCore *lc, char *result){
 				char *tmp;
 				inaddr.s_addr=ntohl(mapped.addr);
 				tmp=inet_ntoa(inaddr);
+				*port=ntohs(mapped.port);
 				strncpy(result,tmp,LINPHONE_IPADDR_SIZE);
 				if (lc->vtable.display_status!=NULL)
 					lc->vtable.display_status(lc,_("Stun lookup done..."));
-				ms_message("Stun server says we have address %s",result);
+				ms_message("Stun server says we have address %s:i",result,*port);
 				return TRUE;
 			}else{
 				ms_warning("stun lookup failed.");
@@ -637,7 +709,6 @@ static bool_t stun_get_localip(LinphoneCore *lc, char *result){
 
 /*result must be an array of chars at least LINPHONE_IPADDR_SIZE */
 void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result){
-	char *tmp=NULL;
 	if (lc->apply_nat_settings){
 		apply_nat_settings(lc);
 		lc->apply_nat_settings=FALSE;
@@ -650,28 +721,23 @@ void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result
 		if (lc->sip_conf.ipv6_enabled){
 			ms_warning("stun support is not implemented for ipv6");
 		}else{
+			int mport=0;
 			ms_message("doing stun lookup for local address...");
-			if (stun_get_localip(lc,result)){
+			if (stun_get_localip(lc,result,&mport)){
 				if (!lc->net_conf.nat_sdp_only)
-					eXosip_set_firewallip(result);
+					eXosip_masquerade_contact(result,mport);
 				return;
 			}
 			ms_warning("stun lookup failed, falling back to a local interface...");
 		}
 		
 	}
-	if (dest==NULL){
-		/* try a public address */
-		if (!lc->sip_conf.ipv6_enabled){
-			dest="15.128.128.93";
-		}else{
-			dest="3ffe:4015:bbbb:70d0:201:2ff:fe09:81b1";
-		}
-	}
-	eXosip_get_localip_for((char*)dest,&tmp);
-	eXosip_set_firewallip("");
-	strncpy(result,tmp,LINPHONE_IPADDR_SIZE);
-	osip_free(tmp);
+	if (eXosip_guess_localip(lc->sip_conf.ipv6_enabled ? AF_INET6 : AF_INET,result,LINPHONE_IPADDR_SIZE)<0){
+		/*default to something */
+		strncpy(result,lc->sip_conf.ipv6_enabled ? "::1" : "127.0.0.1",LINPHONE_IPADDR_SIZE);
+		ms_error("Could not find default routable ip address !"); 
+	}	
+	eXosip_masquerade_contact("",0);
 }
 
 const char *linphone_core_get_primary_contact(LinphoneCore *lc)
@@ -804,11 +870,20 @@ static bool_t exosip_running=FALSE;
 
 void linphone_core_set_sip_port(LinphoneCore *lc,int port)
 {
+	const char *anyaddr;
+	char ua_string[256];
 	int err=0;
 	if (port==lc->sip_conf.sip_port) return;
 	lc->sip_conf.sip_port=port;
 	if (exosip_running) eXosip_quit();
-	err=eXosip_init(NULL,stdout,port);
+	eXosip_init();
+	eXosip_enable_ipv6(lc->sip_conf.ipv6_enabled);
+	if (lc->sip_conf.ipv6_enabled)
+		anyaddr="::0";
+	else
+		anyaddr="0.0.0.0";
+	err=eXosip_listen_addr (IPPROTO_UDP, anyaddr, port,
+		lc->sip_conf.ipv6_enabled ?  PF_INET6 : PF_INET, 0);
 	if (err<0){
 		char *msg=ortp_strdup_printf("UDP port %i seems already in use ! Cannot initialize.",port);
 		ms_warning(msg);
@@ -821,7 +896,14 @@ void linphone_core_set_sip_port(LinphoneCore *lc,int port)
 	eXosip_set_rsvp_mode (lc->rsvp_enable);
 	eXosip_set_rpc_mode (lc->rpc_enable);
 #endif
-	eXosip_set_user_agent("Linphone-" LINPHONE_VERSION "/eXosip");
+	snprintf(ua_string,sizeof(ua_string),"Linphone/%s (eXosip2/%s)",LINPHONE_VERSION,
+#ifdef HAVE_EXOSIP_GET_VERSION
+		 eXosip_get_version()
+#else
+		 "unknown"
+#endif
+);
+	eXosip_set_user_agent(ua_string);
 	exosip_running=TRUE;
 }
 
@@ -831,7 +913,6 @@ bool_t linphone_core_ipv6_enabled(LinphoneCore *lc){
 void linphone_core_enable_ipv6(LinphoneCore *lc, bool_t val){
 	if (lc->sip_conf.ipv6_enabled!=val){
 		lc->sip_conf.ipv6_enabled=val;
-		eXosip_enable_ipv6(val);
 		if (exosip_running){
 			/* we need to restart eXosip */
 			linphone_core_set_sip_port(lc, lc->sip_conf.sip_port);
@@ -867,8 +948,7 @@ void linphone_core_iterate(LinphoneCore *lc)
 		while((ev=eXosip_event_wait(0,0))!=NULL){
 			linphone_core_process_event(lc,ev);
 		}
-		linphone_core_update_proxy_register(lc);
-		linphone_core_refresh_subscribes(lc);
+		if (lc->automatic_action==0) eXosip_automatic_action();
 	}
 	if (lc->call!=NULL){
 		LinphoneCall *call=lc->call;
@@ -920,15 +1000,17 @@ static osip_to_t *osip_to_create(const char *to){
 	return ret;
 }
 
-bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **real_url, osip_to_t **real_parsed_url){
+bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **real_url, osip_to_t **real_parsed_url, char **route){
 	enum_lookup_res_t *enumres=NULL;
 	osip_to_t *parsed_url=NULL;
 	char *enum_domain=NULL;
 	LinphoneProxyConfig *proxy;
 	char *tmpurl;
-	
+	const char *tmproute;
 	if (real_url!=NULL) *real_url=NULL;
 	if (real_parsed_url!=NULL) *real_parsed_url=NULL;
+	*route=NULL;
+	tmproute=linphone_core_get_route(lc);
 	
 	if (is_enum(url,&enum_domain)){
 		lc->vtable.display_status(lc,_("Looking for telephone number destination..."));
@@ -942,6 +1024,7 @@ bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **rea
 		if (real_url!=NULL) *real_url=ms_strdup(tmpurl);
 		if (real_parsed_url!=NULL) *real_parsed_url=osip_to_create(tmpurl);
 		enum_lookup_res_free(enumres);
+		if (tmproute) *route=ms_strdup(tmproute);
 		return TRUE;
 	}
 	/* check if we have a "sip:" */
@@ -949,22 +1032,25 @@ bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **rea
 		/* this doesn't look like a true sip uri */
 		proxy=lc->default_proxy;
 		if (proxy!=NULL){
-			/* append the proxy suffix */
-			osip_uri_t *proxy_uri;
+			/* append the proxy domain suffix */
+			osip_from_t *uri;
 			char *sipaddr;
-			const char *proxy_addr=linphone_proxy_config_get_addr(proxy);
-			osip_uri_init(&proxy_uri);
-			if (osip_uri_parse(proxy_uri,proxy_addr)<0){
-				osip_uri_free(proxy_uri);
+			const char *identity=linphone_proxy_config_get_identity(proxy);
+			osip_from_init(&uri);
+			if (osip_from_parse(uri,identity)<0){
+				osip_from_free(uri);
 				return FALSE;
 			}
-			if (proxy_uri->port!=NULL)
-				sipaddr=ortp_strdup_printf("sip:%s@%s:%s",url,proxy_uri->host,proxy_uri->port);
-			else
-				sipaddr=ortp_strdup_printf("sip:%s@%s",url,proxy_uri->host);
+			sipaddr=ortp_strdup_printf("sip:%s@%s",url,uri->url->host);
 			if (real_parsed_url!=NULL) *real_parsed_url=osip_to_create(sipaddr);
 			if (real_url!=NULL) *real_url=sipaddr;
 			else ms_free(sipaddr);
+			/*if the prompted uri was auto-suffixed with proxy domain,
+			then automatically set a route so that the request goes
+			through the proxy*/
+			if (tmproute==NULL) 
+				*route=ms_strdup(linphone_proxy_config_get_addr(proxy));
+			else *route=ms_strdup(tmproute);
 			return TRUE;
 		}
 	}
@@ -973,6 +1059,7 @@ bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **rea
 		if (real_url!=NULL) *real_url=ms_strdup(url);
 		if (real_parsed_url!=NULL) *real_parsed_url=parsed_url;
 		else osip_to_free(parsed_url);
+		if (tmproute) *route=ms_strdup(tmproute);
 		return TRUE;
 	}
 	/* else we could not do anything with url given by user, so display an error */
@@ -1002,6 +1089,15 @@ const char * linphone_core_get_route(LinphoneCore *lc){
 	return route;
 }
 
+void linphone_set_sdp(osip_message_t *sip, const char *sdpmesg){
+	int sdplen=strlen(sdpmesg);
+	char clen[10];
+	snprintf(clen,sizeof(clen),"%i",sdplen);
+	osip_message_set_body(sip,sdpmesg,sdplen);
+	osip_message_set_content_type(sip,"application/sdp");
+	osip_message_set_content_length(sip,clen);
+}
+
 int linphone_core_invite(LinphoneCore *lc, const char *url)
 {
 	char *barmsg;
@@ -1018,11 +1114,12 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	
 	if (lc->call!=NULL){
 		lc->vtable.display_warning(lc,_("Sorry, having multiple simultaneous calls is not supported yet !"));
+		return -1;
 	}
 
 	gstate_new_state(lc, GSTATE_CALL_OUT_INVITE, url);
 	linphone_core_get_default_proxy(lc,&proxy);
-	if (!linphone_core_interpret_url(lc,url,&real_url,&real_parsed_url)){
+	if (!linphone_core_interpret_url(lc,url,&real_url,&real_parsed_url,&route)){
 		/* bad url */
 		gstate_new_state(lc, GSTATE_CALL_ERROR, NULL);
 		return -1;
@@ -1031,30 +1128,35 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	lc->vtable.display_status(lc,barmsg);
 	ms_free(barmsg);
 	if (proxy!=NULL) {
-		route=linphone_proxy_config_get_route(proxy);
 		from=linphone_proxy_config_get_identity(proxy);
+		
 	}
 	/* if no proxy or no identity defined for this proxy, default to primary contact*/
 	if (from==NULL) from=linphone_core_get_primary_contact(lc);
 
-	err=eXosip_build_initial_invite(&invite,(char*)real_url,(char*)from,
-								(char*)route,"Phone call");
+	err=eXosip_call_build_initial_invite(&invite,real_url,from,
+						route,"Phone call");
 
 	if (err<0){
 		ms_warning("Could not build initial invite");
 		goto end;
 	}
+	
 	/* make sdp message */
 	
 	osip_from_init(&parsed_url2);
 	osip_from_parse(parsed_url2,from);
 	
 	lc->call=linphone_call_new_outgoing(lc,parsed_url2,real_parsed_url);
-	ctx=lc->call->sdpctx;
-	sdpmesg=sdp_context_get_offer(ctx);
+	if (!lc->sip_conf.sdp_200_ack){
+		ctx=lc->call->sdpctx;
+		lc->call->profile=lc->local_profile;
+		sdpmesg=sdp_context_get_offer(ctx);
+		linphone_set_sdp(invite,sdpmesg);
+		linphone_core_init_media_streams(lc);
+	}
 	eXosip_lock();
-	linphone_core_init_media_streams(lc);
-	err=eXosip_initiate_call_with_body(invite,"application/sdp",sdpmesg,(void*)lc->call);
+	err=eXosip_call_send_initial_invite(invite);
 	lc->call->cid=err;
 	eXosip_unlock();
 	if (err<0){
@@ -1072,36 +1174,40 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 		if (parsed_url2!=NULL) osip_from_free(parsed_url2);
 		if (err<0)
 			gstate_new_state(lc, GSTATE_CALL_ERROR, NULL);
+		if (route!=NULL) ms_free(route);
 	return (err<0) ? -1 : 0;
 }
 
 int linphone_core_refer(LinphoneCore *lc, const char *url)
 {
-    char *real_url=NULL;
-    osip_to_t *real_parsed_url=NULL;
-    LinphoneCall *call;
-    if (!linphone_core_interpret_url(lc,url,&real_url,&real_parsed_url)){
-        /* bad url */
-        return -1;
-    }
-    call=lc->call;
-    if (call==NULL){
-        ms_warning("No established call to refer.");
-        return -1;
-    }
-    lc->call=NULL;
-    
-    eXosip_lock();
-    eXosip_transfer_call(call->did, real_url);
-    eXosip_unlock();
-    return 0;
-}    
+	char *real_url=NULL;
+	osip_to_t *real_parsed_url=NULL;
+	LinphoneCall *call;
+	osip_message_t *msg=NULL;
+	char *route;
+	if (!linphone_core_interpret_url(lc,url,&real_url,&real_parsed_url, &route)){
+		/* bad url */
+		return -1;
+	}
+	if (route!=NULL) ms_free(route);
+	call=lc->call;
+	if (call==NULL){
+		ms_warning("No established call to refer.");
+		return -1;
+	}
+	lc->call=NULL;
+	eXosip_call_build_refer(call->did, real_url, &msg);
+	eXosip_lock();
+	eXosip_call_send_request(call->did, msg);
+	eXosip_unlock();
+	return 0;
+}
 
 bool_t linphone_core_inc_invite_pending(LinphoneCore*lc){
-	if (lc->call==NULL){
-		return FALSE;
+	if (lc->call!=NULL && lc->call->dir==LinphoneCallIncoming){
+		return TRUE;
 	}
-	return TRUE;
+	return FALSE;
 }
 
 #ifdef VINCENT_MAURY_RSVP
@@ -1236,14 +1342,23 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 			video_preview_stop(lc->previewstream);
 			lc->previewstream=NULL;
 		}
-		if (lc->video_conf.enabled){
+		if (lc->video_conf.display || lc->video_conf.capture) {
 			StreamParams *video_params=&call->video_params;
 			
 			if (video_params->remoteport>0){
-				
-				video_stream_start(lc->videostream, call->profile,
-				video_params->remoteaddr,video_params->remoteport,
-					video_params->pt,jitt_comp, lc->video_conf.device);
+				if (lc->video_conf.display && lc->video_conf.capture)
+					video_stream_start(lc->videostream,
+					call->profile, video_params->remoteaddr, video_params->remoteport,
+					video_params->pt, jitt_comp, lc->video_conf.device);
+				else if (lc->video_conf.display)
+					video_stream_recv_only_start(lc->videostream,
+					call->profile, video_params->remoteaddr, video_params->remoteport,
+					video_params->pt, jitt_comp, lc->video_conf.device);
+				else if (lc->video_conf.capture)
+					video_stream_send_only_start(lc->videostream,
+					call->profile, video_params->remoteaddr, video_params->remoteport, 
+					video_params->pt, jitt_comp, lc->video_conf.device);				
+
 				video_stream_set_rtcp_information(lc->videostream, cname,tool);
 			}
 		}
@@ -1263,7 +1378,12 @@ void linphone_core_stop_media_streams(LinphoneCore *lc){
 	}
 #ifdef VIDEO_ENABLED
 	if (lc->videostream!=NULL){
-		video_stream_stop(lc->videostream);
+		if (lc->video_conf.display && lc->video_conf.capture)
+			video_stream_stop(lc->videostream);
+		else if (lc->video_conf.display)
+			video_stream_recv_only_stop(lc->videostream);
+		else if (lc->video_conf.capture)
+			video_stream_send_only_stop(lc->videostream);
 		lc->videostream=NULL;
 	}
 	if (linphone_core_video_preview_enabled(lc)){
@@ -1277,8 +1397,12 @@ void linphone_core_stop_media_streams(LinphoneCore *lc){
 int linphone_core_accept_call(LinphoneCore *lc, const char *url)
 {
 	char *sdpmesg;
+	osip_message_t *msg=NULL;
+	LinphoneCall *call=lc->call;
+	int err;
+	bool_t offering=FALSE;
 	
-	if (lc->call==NULL){
+	if (call==NULL){
 		return -1;
 	}
 	
@@ -1286,21 +1410,51 @@ int linphone_core_accept_call(LinphoneCore *lc, const char *url)
 		/*call already accepted*/
 		return -1;
 	}
-	
+
 	/*stop ringing */
 	if (lc->ringstream!=NULL) {
+		ms_message("stop ringing");
 		ring_stop(lc->ringstream);
+		ms_message("ring stopped");
 		lc->ringstream=NULL;
 	}
 	/* sends a 200 OK */
-	sdpmesg=lc->call->sdpctx->answerstr;	/* takes the sdp already computed*/
+	err=eXosip_call_build_answer(call->tid,200,&msg);
+	if (err<0 || msg==NULL){
+		ms_error("Fail to build answer for call: err=%i",err);
+		return -1;
+	}
+	ms_message("eXosip_call_build_answer done");
+	/*if a sdp answer is computed, send it, else send an offer */
+	sdpmesg=call->sdpctx->answerstr;
+	if (sdpmesg==NULL){
+		offering=TRUE;
+		call->profile=lc->local_profile;
+		ms_message("generating sdp offer");
+		sdpmesg=sdp_context_get_offer(call->sdpctx);
+		
+		if (sdpmesg==NULL){
+			ms_error("fail to generate sdp offer !");
+			return -1;
+		}
+		ms_message("sdp message generated (sdpmesg=%p):\n%s",sdpmesg,sdpmesg);
+		linphone_set_sdp(msg,sdpmesg);
+		ms_message("sdp message attached to SIP answer");
+		linphone_core_init_media_streams(lc);
+		ms_message("init_media_streams done");
+	}else{
+		linphone_set_sdp(msg,sdpmesg);
+	}
 	eXosip_lock();
-	eXosip_answer_call_with_body(lc->call->did,200,"application/sdp",sdpmesg);
+	ms_message("eXosip_lock() done");
+	eXosip_call_send_answer(call->tid,200,msg);
+	ms_message("SIP answer sent.");
 	eXosip_unlock();
 	lc->vtable.display_status(lc,_("Connected."));
 	gstate_new_state(lc, GSTATE_CALL_IN_CONNECTED, NULL);
 	
-	linphone_core_start_media_streams(lc, lc->call);
+	if (!offering) linphone_core_start_media_streams(lc, lc->call);
+	ms_message("call answered.");
 	return 0;
 }
 
@@ -1313,7 +1467,7 @@ int linphone_core_terminate_call(LinphoneCore *lc, const char *url)
 	lc->call=NULL;
 	
 	eXosip_lock();
-	eXosip_terminate_call(call->cid,call->did);
+	eXosip_call_terminate(call->cid,call->did);
 	eXosip_unlock();
 	
 	/*stop ringing*/
@@ -1376,6 +1530,7 @@ void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,
 		 */
 		linphone_core_send_publish(lc,presence_mode);
 	}
+	lc->prev_mode=lc->presence_mode;
 	lc->presence_mode=presence_mode;
 	
 }
@@ -1414,27 +1569,34 @@ void linphone_core_set_rec_level(LinphoneCore *lc, int level)
 	if (sndcard) ms_snd_card_set_level(sndcard,MS_SND_CARD_CAPTURE,level);
 }
 
-static MSSndCard *get_card_from_string_id(const char *devid){
+static MSSndCard *get_card_from_string_id(const char *devid, unsigned int cap){
 	MSSndCard *sndcard=NULL;
 	if (devid!=NULL){
 		sndcard=ms_snd_card_manager_get_card(ms_snd_card_manager_get(),devid);
+		if (sndcard!=NULL && 
+			(ms_snd_card_get_capabilities(sndcard) & cap)==0 ){
+			ms_warning("%s card does not have the %s capability, ignoring.",
+				devid,
+				cap==MS_SND_CARD_CAP_CAPTURE ? "capture" : "playback");
+			sndcard=NULL;
+		}
 	}
 	if (sndcard==NULL) sndcard=ms_snd_card_manager_get_default_card(ms_snd_card_manager_get());
 	return sndcard;
 }
 
 int linphone_core_set_ringer_device(LinphoneCore *lc, const char * devid){
-	lc->sound_conf.ring_sndcard=get_card_from_string_id(devid);
+	lc->sound_conf.ring_sndcard=get_card_from_string_id(devid,MS_SND_CARD_CAP_PLAYBACK);
 	return 0;
 }
 
 int linphone_core_set_playback_device(LinphoneCore *lc, const char * devid){
-	lc->sound_conf.play_sndcard=get_card_from_string_id(devid);
+	lc->sound_conf.play_sndcard=get_card_from_string_id(devid,MS_SND_CARD_CAP_PLAYBACK);
 	return 0;
 }
 
 int linphone_core_set_capture_device(LinphoneCore *lc, const char * devid){
-	lc->sound_conf.capt_sndcard=get_card_from_string_id(devid);
+	lc->sound_conf.capt_sndcard=get_card_from_string_id(devid,MS_SND_CARD_CAP_CAPTURE);
 	return 0;
 }
 
@@ -1532,26 +1694,31 @@ bool_t linphone_core_echo_cancelation_enabled(LinphoneCore *lc){
 
 void linphone_core_send_dtmf(LinphoneCore *lc,char dtmf)
 {
-  if (linphone_core_get_use_info_for_dtmf(lc)==0){
-    /* In Band DTMF */
-    if (lc->audiostream!=NULL){
-      audio_stream_send_dtmf(lc->audiostream,dtmf);
-    }
-  }
-  else{
-	char dtmf_body[1000];
-    /* Out of Band DTMF (use INFO method) */
-    LinphoneCall *call=lc->call;
-    if (call==NULL){
-      return;
-    }
-    
-    
-    snprintf(dtmf_body, 999, "Signal=%c\r\nDuration=250\r\n", dtmf);
-    eXosip_lock();
-    eXosip_info_call(call->did, "application/dtmf-relay", dtmf_body);
-    eXosip_unlock();
-  }
+	if (linphone_core_get_use_info_for_dtmf(lc)==0){
+		/* In Band DTMF */
+		if (lc->audiostream!=NULL){
+			audio_stream_send_dtmf(lc->audiostream,dtmf);
+		}
+	}else{
+		char dtmf_body[1000];
+		char clen[10];
+		osip_message_t *msg=NULL;
+		/* Out of Band DTMF (use INFO method) */
+		LinphoneCall *call=lc->call;
+		if (call==NULL){
+			return;
+		}
+		eXosip_call_build_info(call->did,&msg);
+		snprintf(dtmf_body, 999, "Signal=%c\r\nDuration=250\r\n", dtmf);
+		osip_message_set_body(msg,dtmf_body,strlen(dtmf_body));
+		osip_message_set_content_type(msg,"application/dtmf-relay");
+		snprintf(clen,sizeof(clen),"%i",strlen(dtmf_body));
+		osip_message_set_content_length(msg,clen);
+		
+		eXosip_lock();
+		eXosip_call_send_request(call->did,msg);
+		eXosip_unlock();
+	}
 }
 
 void linphone_core_set_stun_server(LinphoneCore *lc, const char *server){
@@ -1616,14 +1783,14 @@ static void apply_nat_settings(LinphoneCore *lc){
 	if (lc->net_conf.firewall_policy==LINPHONE_POLICY_USE_NAT_ADDRESS){
 		if (tmp!=NULL){
 			if (!lc->net_conf.nat_sdp_only)
-				eXosip_set_firewallip(tmp);
+				eXosip_masquerade_contact(tmp,lc->sip_conf.sip_port);
 			ms_free(tmp);
 		}
 		else 
-			eXosip_set_firewallip("");
+			eXosip_masquerade_contact("",0);
 	}
 	else {
-		eXosip_set_firewallip("");	
+		eXosip_masquerade_contact("",0);	
 	}
 }
 
@@ -1674,13 +1841,18 @@ static void toggle_video_preview(LinphoneCore *lc, bool_t val){
 #endif
 }
 
-void linphone_core_enable_video(LinphoneCore *lc, bool_t val){
+void linphone_core_enable_video(LinphoneCore *lc, bool_t vcap_enabled, bool_t display_enabled){
 #ifndef VIDEO_ENABLED
-	if (val)
+	if (vcap_enabled || display_enabled)
 		ms_warning("This version of linphone was built without video support.");
 #endif
-	lc->video_conf.enabled=val;
-	lc->video_conf.show_local=val;
+	lc->video_conf.capture=vcap_enabled;
+	lc->video_conf.display=display_enabled;
+	if (vcap_enabled && display_enabled)
+		lc->video_conf.show_local=1;
+	else
+		lc->video_conf.show_local=0;
+
 	/* need to re-apply network bandwidth settings*/
 	linphone_core_set_download_bandwidth(lc,
 		linphone_core_get_download_bandwidth(lc));
@@ -1689,7 +1861,7 @@ void linphone_core_enable_video(LinphoneCore *lc, bool_t val){
 }
 
 bool_t linphone_core_video_enabled(LinphoneCore *lc){
-	return lc->video_conf.enabled;
+	return (lc->video_conf.display || lc->video_conf.capture);
 }
 
 
@@ -1780,6 +1952,7 @@ void sip_config_uninit(LinphoneCore *lc)
 		while((ev=eXosip_event_wait(0,0))!=NULL){
 		  linphone_core_process_event(lc,ev);
 		}
+		eXosip_automatic_action();
 #ifndef WIN32
 		usleep(100000);
 #else
@@ -1830,7 +2003,8 @@ void sound_config_uninit(LinphoneCore *lc)
 void video_config_uninit(LinphoneCore *lc)
 {
 	video_config_t *config=&lc->video_conf;
-	lp_config_set_int(lc->config,"video","enabled",config->enabled);
+	lp_config_set_int(lc->config,"video","display",config->display);
+	lp_config_set_int(lc->config,"video","capture",config->capture);
 	lp_config_set_int(lc->config,"video","show_local",config->show_local);
 }
 
