@@ -26,13 +26,14 @@
 #define _PRIVATE_H
 
 #include "linphonecore.h"
+#include <eXosip2/eXosip.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #ifndef LINPHONE_VERSION
-#define LINPHONE_VERSION "1.6.1"
+#define LINPHONE_VERSION "2.0.0"
 #endif
 
 #ifndef LIBLINPHONE_VERSION 
@@ -67,9 +68,9 @@ int linphone_proxy_config_send_publish(LinphoneProxyConfig *cfg, LinphoneOnlineS
 
 int linphone_online_status_to_eXosip(LinphoneOnlineStatus os);
 
-void linphone_friend_set_sid(LinphoneFriend *lf, int cid);
-void linphone_friend_set_nid(LinphoneFriend *lf, int cid);
-void linphone_friend_notify(LinphoneFriend *lf, int ss, int os);
+void linphone_friend_set_sid(LinphoneFriend *lf, int sid);
+void linphone_friend_set_nid(LinphoneFriend *lf, int nid);
+void linphone_friend_notify(LinphoneFriend *lf, int ss, LinphoneOnlineStatus os);
 
 int set_lock_file();
 int get_lock_file();
@@ -98,4 +99,21 @@ static inline bool_t bandwidth_is_greater(int bw1, int bw2){
 
 #define PAYLOAD_TYPE_ENABLED	PAYLOAD_TYPE_USER_FLAG_0
 
+void linphone_process_authentication(LinphoneCore* lc, eXosip_event_t *ev);
+void linphone_authentication_ok(LinphoneCore *lc, eXosip_event_t *ev);
+void linphone_subscription_new(LinphoneCore *lc, eXosip_event_t *ev);
+void linphone_notify_recv(LinphoneCore *lc,eXosip_event_t *ev);
+LinphoneProxyConfig *linphone_core_get_proxy_config_from_rid(LinphoneCore *lc, int rid);
+
+void linphone_subscription_answered(LinphoneCore *lc, eXosip_event_t *ev);
+void linphone_subscription_closed(LinphoneCore *lc, eXosip_event_t *ev);
+
+void linphone_call_init_media_params(LinphoneCall *call);
+
+void linphone_set_sdp(osip_message_t *sip, const char *sdp);
+
+MSList *find_friend(MSList *fl, const osip_from_t *friend, LinphoneFriend **lf);
+LinphoneFriend *linphone_find_friend_by_nid(MSList *l, int nid);
+LinphoneFriend *linphone_find_friend_by_sid(MSList *l, int sid);
+
 #endif /* _PRIVATE_H */
