@@ -25,6 +25,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/mssndcard.h"
 
+/*uncomment the following line if you have problems with an alsa driver
+having sound quality trouble:*/
+/*#define EPIPE_BUGFIX 1*/
+
 static MSSndCard * alsa_card_new(int id);
 static MSSndCard *alsa_card_duplicate(MSSndCard *obj);
 static MSFilter * ms_alsa_read_new(const char *dev);
@@ -140,6 +144,38 @@ static int alsa_set_params(snd_pcm_t *pcm_handle, int rw, int bits, int stereo,
 	return 0;	
 }
 
+#ifdef EPIPE_BUGFIX
+static void alsa_fill_w (snd_pcm_t *pcm_handle)
+{
+	snd_pcm_hw_params_t *hwparams=NULL;
+	int channels;
+        snd_pcm_uframes_t buffer_size;
+	int buffer_size_bytes;
+	void *buffer;
+
+	/* Allocate the snd_pcm_hw_params_t structure on the stack. */
+	snd_pcm_hw_params_alloca(&hwparams);
+	snd_pcm_hw_params_current(pcm_handle, hwparams);
+
+	/* get channels */
+	snd_pcm_hw_params_get_channels (hwparams, &channels);
+
+	/* get buffer size */
+	snd_pcm_hw_params_get_buffer_size (hwparams, &buffer_size);
+
+	/* fill half */
+	buffer_size /= 2;
+
+	/* allocate buffer assuming 2 bytes per sample */
+	buffer_size_bytes = buffer_size * channels * 2;
+	buffer = alloca (buffer_size_bytes);
+	memset (buffer, 0, buffer_size_bytes);
+
+	/* write data */
+	snd_pcm_writei(pcm_handle, buffer, buffer_size);
+}
+#endif
+
 static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 {
 	snd_pcm_t *pcm_handle;
@@ -149,7 +185,7 @@ static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 
 
 	if (snd_pcm_open(&pcm_handle, pcmdev,SND_PCM_STREAM_CAPTURE,SND_PCM_NONBLOCK) < 0) {
-		ms_warning("alsa_card_open_r: Error opening PCM device %s\n",pcmdev );
+		ms_warning("alsa_open_r: Error opening PCM device %s\n",pcmdev );
 		return NULL;
 	}
 	if (alsa_set_params(pcm_handle,0,bits,stereo,rate)<0){
@@ -168,7 +204,7 @@ static snd_pcm_t * alsa_open_w(const char *pcmdev,int bits,int stereo,int rate)
 	snd_pcm_t *pcm_handle;
 	
 	if (snd_pcm_open(&pcm_handle, pcmdev,SND_PCM_STREAM_PLAYBACK,SND_PCM_NONBLOCK) < 0) {
-		ms_warning("alsa_card_open_r: Error opening PCM device %s\n",pcmdev );
+		ms_warning("alsa_open_w: Error opening PCM device %s\n",pcmdev );
 		return NULL;
 	}
 	if (alsa_set_params(pcm_handle,1,bits,stereo,rate)<0){
@@ -204,13 +240,16 @@ static int alsa_write(snd_pcm_t *handle,unsigned char *buf,int nsamples)
 	if ((err=snd_pcm_writei(handle,buf,nsamples))<0){
 		if (err==-EPIPE){
 			snd_pcm_prepare(handle);
+#ifdef EPIPE_BUGFIX
+			alsa_fill_w (handle);
+#endif
 			err=snd_pcm_writei(handle,buf,nsamples);
 			if (err<0) ms_warning("alsa_card_write: Error writing sound buffer (nsamples=%i):%s",nsamples,snd_strerror(err));
 		}else if (err!=-EWOULDBLOCK){
 			ms_warning("alsa_card_write: snd_pcm_writei() failed:%s.",snd_strerror(err));
 		}
 	}else if (err!=nsamples) {
-		ms_warning("Only %i samples written instead of %i",err,nsamples);
+		ms_debug("Only %i samples written instead of %i",err,nsamples);
 	}
 	return err;
 }
@@ -662,6 +701,9 @@ void alsa_write_process(MSFilter *obj){
 	int err;
 	if (ad->handle==NULL && ad->pcmdev!=NULL){
 		ad->handle=alsa_open_w(ad->pcmdev,16,ad->nchannels==2,ad->rate);
+#ifdef EPIPE_BUGFIX
+		alsa_fill_w (ad->pcmdev);
+#endif
 	}
 	if (ad->handle==NULL) {
 		ms_queue_flush(obj->inputs[0]);
