@@ -1,24 +1,22 @@
 /*
-  The mediastreamer library aims at providing modular media processing and I/O
-        for linphone, but also for any telephony application.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-                                                                                
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
+mediastreamer2 library - modular sound and video processing and streaming
+Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
 
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
 
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
@@ -80,7 +78,7 @@ void video_stream_change_decoder(VideoStream *stream, int payload){
 			ms_filter_destroy(stream->decoder);
 			stream->decoder=dec;
 			if (pt->recv_fmtp!=NULL)
-				ms_filter_call_method(stream->decoder,MS_FILTER_SET_FMTP,(void*)pt->recv_fmtp);
+				ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
 			ms_filter_link (stream->rtprecv, 0, stream->decoder, 0);
 			ms_filter_link (stream->decoder,0 , stream->sizeconv, 0);
 			ms_filter_preprocess(stream->decoder,stream->ticker);
@@ -103,7 +101,7 @@ static void payload_type_changed(RtpSession *session, unsigned long data){
 }
 
 VideoStream *video_stream_new(int locport, bool_t use_ipv6){
-	VideoStream *stream = ms_new0 (VideoStream, 1);
+	VideoStream *stream = (VideoStream *)ms_new0 (VideoStream, 1);
 	stream->session=create_duplex_rtpsession(locport,use_ipv6);
 	return stream;
 }
@@ -113,9 +111,12 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	PayloadType *pt;
 	RtpSession *rtps=stream->session;
 	MSPixFmt format;
-	MSVideoSize vsize=MS_VIDEO_SIZE_CIF;
+	MSVideoSize vsize;
 	float fps=15;
 
+	vsize.height=MS_VIDEO_SIZE_CIF_H;
+	vsize.width=MS_VIDEO_SIZE_CIF_W;
+
 	pt=rtp_profile_get_payload(profile,payload);
 	if (pt==NULL){
 		ms_error("videostream.c: undefined payload type.");
@@ -183,13 +184,16 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	format=MS_YUV420P;
 	ms_filter_call_method(stream->decoder,MS_FILTER_SET_PIX_FMT,&format);
 	/*ask the size-converter to always output CIF */
-	vsize=MS_VIDEO_SIZE_CIF;
+
+	vsize.height=MS_VIDEO_SIZE_CIF_H;
+	vsize.width=MS_VIDEO_SIZE_CIF_W;
+
 	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_PIX_FMT,&format);
 	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 	ms_filter_call_method(stream->output,MS_FILTER_SET_PIX_FMT,&format);
 
 	if (pt->recv_fmtp!=NULL)
-		ms_filter_call_method(stream->decoder,MS_FILTER_SET_FMTP,(void*)pt->recv_fmtp);
+		ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
 
 	/* and then connect all */
 	ms_filter_link (stream->source, 0, stream->pixconv, 0);
@@ -210,12 +214,14 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	return 0;
 }
 
-
+void video_stream_send_vfu(VideoStream *stream){
+	if (stream->encoder)
+		ms_filter_call_method_noarg(stream->encoder,MS_FILTER_REQ_VFU);
+}
 
 void
 video_stream_stop (VideoStream * stream)
 {
-
 	if (stream->ticker){
 		ms_ticker_detach(stream->ticker,stream->source);
 		ms_filter_call_method_noarg(stream->source,MS_V4L_STOP);
@@ -245,7 +251,7 @@ void video_stream_set_rtcp_information(VideoStream *st, const char *cname, const
 
 
 VideoStream * video_preview_start(const char *device){
-	VideoStream *stream = ms_new0 (VideoStream, 1);
+	VideoStream *stream = (VideoStream *)ms_new0 (VideoStream, 1);
 	MSPixFmt format;
 	MSVideoSize vsize;
 	vsize.width=MS_VIDEO_SIZE_CIF_W;
@@ -289,41 +295,40 @@ void video_preview_stop(VideoStream *stream){
 }
 
 
-VideoStream * video_stream_send_only_start(RtpProfile *profile, int locport, const char *remip, int remport, int payload, const char *device)
-{
-	VideoStream *stream = video_stream_new(locport,ms_is_ipv6(remip));
-	RtpSession *rtps=stream->session;
+int video_stream_send_only_start(VideoStream* stream, RtpProfile *profile, const char *remip, int remport, int payload, int jitt_comp, const char *device){
 	PayloadType *pt;
 	MSPixFmt format;
-	MSVideoSize vsize=MS_VIDEO_SIZE_CIF;
-	float fps=7;
+	MSVideoSize vsize;
+	RtpSession *rtps=stream->session;
+	float fps=15;
 	
+	vsize.width=MS_VIDEO_SIZE_CIF_W;
+	vsize.height=MS_VIDEO_SIZE_CIF_H;
 
 	rtp_session_set_profile(rtps,profile);
 	if (remport>0) rtp_session_set_remote_addr(rtps,remip,remport);
 	rtp_session_set_payload_type(rtps,payload);
-	rtp_session_set_jitter_compensation(rtps,60);
+	rtp_session_set_jitter_compensation(rtps,jitt_comp);
+	
+	/* creates rtp filter to send streams (remote part) */
+	rtp_session_set_recv_buf_size(rtps,MAX_RTP_SIZE);
+	stream->rtpsend =ms_filter_new(MS_RTP_SEND_ID);
+	if (remport>0) ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SET_SESSION,stream->session);
 
+	/* creates the filters */
 	pt=rtp_profile_get_payload(profile,payload);
 	if (pt==NULL){
 		video_stream_free(stream);
 		ms_error("videostream.c: undefined payload type.");
-		return NULL;
+		return -1;
 	}
 	stream->encoder=ms_filter_create_encoder(pt->mime_type);
 	if ((stream->encoder==NULL)){
 		/* big problem: we have not a registered codec for this payload...*/
 		video_stream_free(stream);
 		ms_error("videostream.c: No codecs available for payload %i.",payload);
-		return NULL;
+		return -1;
 	}
-	
-	
-	/* creates two rtp filters to recv send streams (remote part) */
-	rtp_session_set_recv_buf_size(stream->session,MAX_RTP_SIZE);
-	stream->rtpsend =ms_filter_new(MS_RTP_SEND_ID);
-	if (remport>0) ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SET_SESSION,stream->session);
-	
 
 	/* creates the filters */
 	stream->source = ms_filter_new(MS_V4L_ID);
@@ -345,8 +350,9 @@ VideoStream * video_stream_send_only_start(RtpProfile *profile, int locport, con
 	/*set it to the pixconv */
 	ms_filter_call_method(stream->pixconv,MS_FILTER_SET_PIX_FMT,&format);
 	ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
-
-	/*force the decoder to output YUV420P */
+	
+	ms_message("vsize=%ix%i, fps=%f, send format: %s, capture format: %d, bitrate: %d",
+			vsize.width,vsize.height,fps,pt->send_fmtp,format, pt->normal_bitrate);
 
 	/* and then connect all */
 	ms_filter_link (stream->source, 0, stream->pixconv, 0);
@@ -357,7 +363,7 @@ VideoStream * video_stream_send_only_start(RtpProfile *profile, int locport, con
 	stream->ticker = ms_ticker_new(); 
 	/* attach it the graph */
 	ms_ticker_attach (stream->ticker, stream->source);
-	return stream;
+	return 0;
 }
 
 void video_stream_send_only_stop(VideoStream *stream){
@@ -369,3 +375,78 @@ void video_stream_send_only_stop(VideoStream *stream){
 	
 	video_stream_free(stream);
 }
+
+int video_stream_recv_only_start (VideoStream *stream, RtpProfile *profile, const char *remip, int remport,int payload, int jitt_comp,  const char *device){
+	PayloadType *pt;
+	MSPixFmt format;
+	MSVideoSize vsize;
+	RtpSession *rtps=stream->session;
+	
+	vsize.width=MS_VIDEO_SIZE_CIF_W;
+	vsize.height=MS_VIDEO_SIZE_CIF_H;
+
+	rtp_session_set_profile(rtps,profile);
+	if (remport>0) rtp_session_set_remote_addr(rtps,remip,remport);
+	rtp_session_set_payload_type(rtps,payload);
+	rtp_session_set_jitter_compensation(rtps,jitt_comp);
+	
+	/* creates rtp filters to recv streams */
+	rtp_session_set_recv_buf_size(rtps,MAX_RTP_SIZE);
+	stream->rtprecv = ms_filter_new (MS_RTP_RECV_ID);
+	ms_filter_call_method(stream->rtprecv,MS_RTP_RECV_SET_SESSION,rtps);
+
+	/* creates the filters */
+	pt=rtp_profile_get_payload(profile,payload);
+	if (pt==NULL){
+		ms_error("videostream.c: undefined payload type.");
+		return -1;
+	}
+	stream->decoder=ms_filter_create_decoder(pt->mime_type);
+	if (stream->decoder==NULL){
+		/* big problem: we have not a registered codec for this payload...*/
+		ms_error("videostream.c: No codecs available for payload %i:%s.",payload,pt->mime_type);
+		return -1;
+	}
+
+	stream->sizeconv = ms_filter_new(MS_SIZE_CONV_ID);
+	stream->output=ms_filter_new(MS_VIDEO_OUT_ID);
+
+	/*force the decoder to output YUV420P */
+	format=MS_YUV420P;
+	/*ask the size-converter to always output CIF */
+	vsize.width=MS_VIDEO_SIZE_CIF_W;
+	vsize.height=MS_VIDEO_SIZE_CIF_H;
+	ms_message("Setting output vsize=%ix%i",vsize.width,vsize.height);
+	
+	ms_filter_call_method(stream->decoder,MS_FILTER_SET_PIX_FMT,&format);
+	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_PIX_FMT,&format);
+	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+	ms_filter_call_method(stream->output,MS_FILTER_SET_PIX_FMT,&format);
+	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+
+	if (pt->recv_fmtp!=NULL) {
+		ms_message("pt->recv_fmtp: %s", pt->recv_fmtp);
+		ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
+	}
+
+	/* and then connect all */
+	ms_filter_link (stream->rtprecv, 0, stream->decoder, 0);
+	ms_filter_link (stream->decoder,0 , stream->sizeconv, 0);
+	ms_filter_link (stream->sizeconv, 0, stream->output, 0);
+
+	/* create the ticker */
+	stream->ticker = ms_ticker_new(); 
+	/* attach it the graph */
+	ms_ticker_attach (stream->ticker, stream->rtprecv);
+	return 0;
+}
+
+void video_stream_recv_only_stop (VideoStream * stream){
+	ms_ticker_detach(stream->ticker, stream->rtprecv);
+	rtp_stats_display(rtp_session_get_stats(stream->session),"Video session's RTP statistics");
+	ms_filter_unlink(stream->rtprecv, 0, stream->decoder, 0);
+	ms_filter_unlink(stream->decoder,0,stream->sizeconv,0);
+	ms_filter_unlink(stream->sizeconv,0,stream->output,0);
+	video_stream_free (stream);
+}
+
