@@ -59,6 +59,7 @@ struct _PayloadType
 	/* other useful information for the application*/
 	int normal_bitrate;	/*in bit/s */
 	char *mime_type; /**<actually the submime, ex: pcm, pcma, gsm*/
+	int channels; /**< number of channels of audio */
 	char *recv_fmtp; /* various format parameters for the incoming stream */
 	char *send_fmtp; /* various format parameters for the outgoing stream */
 	int flags;
@@ -92,9 +93,13 @@ typedef struct _RtpProfile RtpProfile;
 
 PayloadType *payload_type_new(void);
 PayloadType *payload_type_clone(PayloadType *payload);
+char *payload_type_get_rtpmap(PayloadType *pt);
 void payload_type_destroy(PayloadType *pt);
 void payload_type_set_recv_fmtp(PayloadType *pt, const char *fmtp);
 void payload_type_set_send_fmtp(PayloadType *pt, const char *fmtp);
+void payload_type_append_recv_fmtp(PayloadType *pt, const char *fmtp);
+void payload_type_append_send_fmtp(PayloadType *pt, const char *fmtp);
+
 
 bool_t fmtp_get_value(const char *fmtp, const char *param_name, char *result, size_t result_len);
 
@@ -137,8 +142,8 @@ PayloadType * rtp_profile_get_payload_from_mime(RtpProfile *profile,const char *
 PayloadType * rtp_profile_get_payload_from_rtpmap(RtpProfile *profile, const char *rtpmap);
 int rtp_profile_get_payload_number_from_mime(RtpProfile *profile,const char *mime);
 int rtp_profile_get_payload_number_from_rtpmap(RtpProfile *profile, const char *rtpmap);
-int rtp_profile_find_payload_number(RtpProfile *prof,const char *mime,int rate);
-PayloadType * rtp_profile_find_payload(RtpProfile *prof,const char *mime,int rate);
+int rtp_profile_find_payload_number(RtpProfile *prof,const char *mime,int rate, int channels);
+PayloadType * rtp_profile_find_payload(RtpProfile *prof,const char *mime,int rate, int channels);
 int rtp_profile_move_payload(RtpProfile *prof,int oldpos,int newpos);
 
 RtpProfile * rtp_profile_new(const char *name);
@@ -157,10 +162,13 @@ void rtp_profile_destroy(RtpProfile *prof);
 VAR_DECLSPEC PayloadType payload_type_pcmu8000;
 VAR_DECLSPEC PayloadType payload_type_pcma8000;
 VAR_DECLSPEC PayloadType payload_type_pcm8000;
+VAR_DECLSPEC PayloadType payload_type_l16_mono;
+VAR_DECLSPEC PayloadType payload_type_l16_stereo;
 VAR_DECLSPEC PayloadType payload_type_lpc1016;
 VAR_DECLSPEC PayloadType payload_type_g729;
 VAR_DECLSPEC PayloadType payload_type_g7231;
 VAR_DECLSPEC PayloadType payload_type_gsm;
+VAR_DECLSPEC PayloadType payload_type_lpc;
 VAR_DECLSPEC PayloadType payload_type_lpc1015;
 VAR_DECLSPEC PayloadType payload_type_speex_nb;
 VAR_DECLSPEC PayloadType payload_type_speex_wb;
@@ -179,6 +187,7 @@ VAR_DECLSPEC PayloadType payload_type_h263_2000;
 VAR_DECLSPEC PayloadType payload_type_mp4v;
 VAR_DECLSPEC PayloadType payload_type_theora;
 
+
 /* telephone-event */
 VAR_DECLSPEC PayloadType payload_type_telephone_event;
 
