@@ -17,6 +17,16 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/** 
+ * \file rtpsession.h
+ * \brief The RtpSession api
+ *
+ * The RtpSession objects represent a RTP session: once it is configured with 
+ * local and remote network addresses and a payload type is given, it let you send
+ * and recv a media stream.
+**/
+
+
 #ifndef RTPSESSION_H
 #define RTPSESSION_H
 
@@ -39,6 +49,15 @@ typedef enum {
 } RtpSessionMode;
 
 
+/*! Jitter buffer parameters
+*/
+typedef struct _JBParameters{
+	int min_size; /**< in milliseconds*/
+	int nom_size; /**< idem */
+	int max_size; /**< idem */
+	bool_t adaptive;
+} JBParameters;
+
 typedef struct _JitterControl
 {
 	int jitt_comp;   /* the user jitt_comp in miliseconds*/
@@ -52,6 +71,7 @@ typedef struct _JitterControl
 	int corrective_step;
 	int corrective_slide;
 	bool_t adaptive;
+	bool_t enabled;
 } JitterControl;
 
 typedef struct _WaitPoint
@@ -61,10 +81,22 @@ typedef struct _WaitPoint
 	uint32_t time;
 	bool_t wakeup;
 } WaitPoint;
+
+typedef struct _RtpTransport
+{
+	void *data;
+	ortp_socket_t (*t_getsocket)(struct _RtpTransport *t);
+	int  (*t_sendto)(struct _RtpTransport *t, mblk_t *msg , int flags, const struct sockaddr *to, socklen_t tolen);
+	int  (*t_recvfrom)(struct _RtpTransport *t, mblk_t *msg, int flags, struct sockaddr *from, socklen_t *fromlen);
+	struct _RtpSession *session;//<back pointer to the owning session, set by oRTP
+}  RtpTransport;
+
+
 	
 typedef struct _RtpStream
 {
 	ortp_socket_t socket;
+        struct _RtpTransport *tr; 
 	int sockfamily;
 	int max_rq_size;
 	int time_jump;
@@ -113,6 +145,8 @@ typedef struct _RtcpStream
 {
 	ortp_socket_t socket;
 	int sockfamily;
+        struct _RtpTransport *tr; 
+
 	mblk_t *cached_mp;
 #ifdef ORTP_INET6
 	struct sockaddr_storage rem_addr;
@@ -120,23 +154,31 @@ typedef struct _RtcpStream
 	struct sockaddr_in rem_addr;
 #endif
 	int rem_addrlen;
+	bool_t enabled; /*tells whether we can send RTCP packets */
 } RtcpStream;
 
 typedef struct _RtpSession RtpSession;
 
 
-
+/**
+ * An object representing a bi-directional RTP session.
+ * It holds sockets, jitter buffer, various counters (timestamp, sequence numbers...)
+ * Applications SHOULD NOT try to read things within the RtpSession object but use
+ * instead its public API (the rtp_session_* methods) where RtpSession is used as a 
+ * pointer.
+ * rtp_session_new() allocates and initialize a RtpSession.
+**/
 struct _RtpSession
 {
 	RtpSession *next;	/* next RtpSession, when the session are enqueued by the scheduler */
 	int mask_pos;	/* the position in the scheduler mask of RtpSession : do not move this field: it is part of the ABI since the session_set macros use it*/
-	RtpProfile *profile;
-	WaitPoint recv_wp;
-	WaitPoint send_wp;
-	uint32_t send_ssrc;
-	uint32_t recv_ssrc;
-	int send_pt;/* sent payload type */
-	int recv_pt;/* recv payload type */
+        struct {
+	  RtpProfile *profile;
+	  int pt;
+	  int ssrc;
+	  WaitPoint wp;
+	  int telephone_events_pt;	/* the payload type used for telephony events */
+	} snd,rcv;
 	int hw_recv_pt; /* recv payload type before jitter buffer */
 	int recv_buf_size;
 	RtpSignalTable on_ssrc_changed;
@@ -161,7 +203,6 @@ struct _RtpSession
 	struct timeval last_recv_time; /* Time of receiving the RTP/RTCP packet. */
 
 	/* telephony events extension */
-	int telephone_events_pt;	/* the payload type used for telephony events */
 	mblk_t *current_tev;		/* the pending telephony events */
 	mblk_t *sd;
 	queue_t contributing_sources;
@@ -178,26 +219,32 @@ extern "C"
 {
 #endif
 
-/*private */
-void rtp_session_init(RtpSession *session, int mode);
-#define rtp_session_set_flag(session,flag) (session)->flags|=(flag)
-#define rtp_session_unset_flag(session,flag) (session)->flags&=~(flag)
-void rtp_session_uninit(RtpSession *session);
-
 /* public API */
 RtpSession *rtp_session_new(int mode);
 void rtp_session_set_scheduling_mode(RtpSession *session, int yesno);
 void rtp_session_set_blocking_mode(RtpSession *session, int yesno);
-void rtp_session_set_profile(RtpSession *session,RtpProfile *profile);
+void rtp_session_set_profile(RtpSession *session, RtpProfile *profile);
+void rtp_session_set_send_profile(RtpSession *session,RtpProfile *profile);
+void rtp_session_set_recv_profile(RtpSession *session,RtpProfile *profile);
 RtpProfile *rtp_session_get_profile(RtpSession *session);
+RtpProfile *rtp_session_get_send_profile(RtpSession *session);
+RtpProfile *rtp_session_get_recv_profile(RtpSession *session);
 int rtp_session_signal_connect(RtpSession *session,const char *signal_name, RtpCallback cb, unsigned long user_data);
 int rtp_session_signal_disconnect_by_callback(RtpSession *session,const char *signal_name, RtpCallback cb);
 void rtp_session_set_ssrc(RtpSession *session, uint32_t ssrc);
 void rtp_session_set_seq_number(RtpSession *session, uint16_t seq);
 uint16_t rtp_session_get_seq_number(RtpSession *session);
+
+void rtp_session_enable_jitter_buffer(RtpSession *session , bool_t enabled);
+bool_t rtp_session_jitter_buffer_enabled(const RtpSession *session);
+void rtp_session_set_jitter_buffer_params(RtpSession *session, const JBParameters *par);
+void rtp_session_get_jitter_buffer_params(RtpSession *session, JBParameters *par);
+
+/*deprecated jitter control functions*/
 void rtp_session_set_jitter_compensation(RtpSession *session, int milisec);
 void rtp_session_enable_adaptive_jitter_compensation(RtpSession *session, bool_t val);
 bool_t rtp_session_adaptive_jitter_compensation_enabled(RtpSession *session);
+
 void rtp_session_set_time_jump_limit(RtpSession *session, int miliseconds);
 int rtp_session_set_local_addr(RtpSession *session,const char *addr, int port);
 int rtp_session_get_local_port(const RtpSession *session);
@@ -205,6 +252,7 @@ int rtp_session_set_remote_addr(RtpSession *session,const char *addr, int port);
 /* alternatively to the set_remote_addr() and set_local_addr(), an application can give
 a valid socket (potentially connect()ed )to be used by the RtpSession */
 void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd);
+void rtp_session_set_transports(RtpSession *session, RtpTransport *rtptr, RtpTransport *rtcptr);
 
 /*those methods are provided for people who wants to send non-RTP messages using the RTP/RTCP sockets */
 ortp_socket_t rtp_session_get_rtp_socket(const RtpSession *session);
@@ -237,6 +285,8 @@ void rtp_session_set_symmetric_rtp (RtpSession * session, bool_t yesno);
 
 void rtp_session_set_connected_mode(RtpSession *session, bool_t yesno);
 
+void rtp_session_enable_rtcp(RtpSession *session, bool_t yesno);
+
 /*low level recv and send functions */
 mblk_t * rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts);
 mblk_t * rtp_session_create_packet(RtpSession *session,int header_size, const uint8_t *payload, int payload_size);
@@ -295,7 +345,11 @@ mblk_t* rtp_session_create_rtcp_sdes_packet(RtpSession *session);
 void rtp_session_get_last_recv_time(RtpSession *session, struct timeval *tv);
 int rtp_session_bye(RtpSession *session, const char *reason);
 
-
+/*private */
+void rtp_session_init(RtpSession *session, int mode);
+#define rtp_session_set_flag(session,flag) (session)->flags|=(flag)
+#define rtp_session_unset_flag(session,flag) (session)->flags&=~(flag)
+void rtp_session_uninit(RtpSession *session);
 
 #ifdef __cplusplus
 }
