@@ -213,6 +213,7 @@ mblk_t *rtp_getq_permissive(queue_t *q,uint32_t timestamp, int *rejected)
 void
 rtp_session_init (RtpSession * session, int mode)
 {
+	JBParameters jbp;
 	memset (session, 0, sizeof (RtpSession));
 	session->rtp.max_rq_size = 100;/* maximum number of packet allowed to be queued */
 	session->mode = (RtpSessionMode) mode;
@@ -256,25 +257,30 @@ rtp_session_init (RtpSession * session, int mode)
 	rtp_session_set_send_payload_type(session,0);
 	/*sets supposed recv payload type to undefined */
 	rtp_session_set_recv_payload_type(session,-1);
-	rtp_session_set_jitter_compensation(session,RTP_DEFAULT_JITTER_TIME);
-	rtp_session_enable_adaptive_jitter_compensation(session,FALSE);
+	/* configure jitter buffer with working default parameters */
+	jbp.min_size=RTP_DEFAULT_JITTER_TIME;
+	jbp.nom_size=RTP_DEFAULT_JITTER_TIME;
+	jbp.max_size=-1;
+	jbp.adaptive=TRUE;
+	rtp_session_enable_jitter_buffer(session,TRUE);
+	rtp_session_set_jitter_buffer_params(session,&jbp);
 	rtp_session_set_time_jump_limit(session,5000);
+	rtp_session_enable_rtcp(session,TRUE);
 	session->recv_buf_size = UDP_MAX_SIZE;
 	session->symmetric_rtp = FALSE;
 	session->permissive=FALSE;
 }
 
-/**
- *rtp_session_new:
- *@mode: One of the #RtpSessionMode flags.
- *
- *	Creates a new rtp session.
- *  If the session is able to send data (RTP_SESSION_SENDONLY or RTP_SESSION_SENDRECV), then a
- *	random SSRC number is choosed for the outgoing stream.
- *
- *Returns: the newly created rtp session.
-**/
 
+/**
+ * Creates a new rtp session.
+ * If the session is able to send data (RTP_SESSION_SENDONLY or
+ * RTP_SESSION_SENDRECV), then a random SSRC number is choosed for 
+ * the outgoing stream.
+ * @param mode One of the RtpSessionMode flags.	
+ *
+ * @return the newly created rtp session.
+**/
 RtpSession *
 rtp_session_new (int mode)
 {
@@ -285,20 +291,18 @@ rtp_session_new (int mode)
 }
 
 /**
- *rtp_session_set_scheduling_mode:
- *@session: a rtp session.
- *@yesno:	a boolean to indicate the scheduling mode.
- *
- *	Sets the scheduling mode of the rtp session. If @yesno is TRUE, the rtp session is in
+ * Sets the scheduling mode of the rtp session. If @yesno is TRUE, the rtp session is in
  *	the scheduled mode, that means that you can use session_set_select() to block until it's time
  *	to receive or send on this session according to the timestamp passed to the respective functions.
  *  You can also use blocking mode (see rtp_session_set_blocking_mode() ), to simply block within
  *	the receive and send functions.
  *	If @yesno is FALSE, the ortp scheduler will not manage those sessions, meaning that blocking mode 
  *  and the use of session_set_select() for this session are disabled.
+ *@param session a rtp session.
+ *@param yesno 	a boolean to indicate the scheduling mode.
+ *
  *
 **/
-
 void
 rtp_session_set_scheduling_mode (RtpSession * session, int yesno)
 {
@@ -323,10 +327,6 @@ rtp_session_set_scheduling_mode (RtpSession * session, int yesno)
 
 
 /**
- *rtp_session_set_blocking_mode:
- *@session: a rtp session
- *@yesno: a boolean
- *
  *	This function implicitely enables the scheduling mode if yesno is TRUE.
  *	rtp_session_set_blocking_mode() defines the behaviour of the rtp_session_recv_with_ts() and 
  *	rtp_session_send_with_ts() functions. If @yesno is TRUE, rtp_session_recv_with_ts()
@@ -335,6 +335,8 @@ rtp_session_set_scheduling_mode (RtpSession * session, int yesno)
  *	For rtp_session_send_with_ts(), it will block until it is time for the packet to be sent.
  *	If @yesno is FALSE, then the two functions will return immediately.
  *
+ *  @param session a rtp session
+ *  @param yesno a boolean
 **/
 void
 rtp_session_set_blocking_mode (RtpSession * session, int yesno)
@@ -347,15 +349,12 @@ rtp_session_set_blocking_mode (RtpSession * session, int yesno)
 }
 
 /**
- *rtp_session_set_profile:
- *@session: a rtp session
- *@profile: a rtp profile
- *
  *	Set the RTP profile to be used for the session. By default, all session are created by
  *	rtp_session_new() are initialized with the AV profile, as defined in RFC 3551. The application
  *	can set any other profile instead using that function.
  *
- *
+ * @param session a rtp session
+ * @param profile a rtp profile
 **/
 
 void
@@ -366,16 +365,24 @@ rtp_session_set_profile (RtpSession * session, RtpProfile * profile)
 	rtp_session_telephone_events_supported(session);
 }
 
+/**
+ *	By default oRTP automatically sends RTCP SR or RR packets. If
+ *	yesno is set to FALSE, the RTCP sending of packet is disabled.
+ *	This functionnality might be needed for some equipments that do not
+ *	support RTCP, leading to a traffic of ICMP errors on the network.
+ *	It can also be used to save bandwidth despite the RTCP bandwidth is 
+ *	actually and usually very very low.
+**/
+void rtp_session_enable_rtcp(RtpSession *session, bool_t yesno){
+	session->rtcp.enabled=yesno;
+}
 
 /**
- *rtp_session_set_send_profile:
- *@session: a rtp session
- *@profile: a rtp profile
- *
  *	Set the RTP profile to be used for the sending by this session. By default, all session are created by
  *	rtp_session_new() are initialized with the AV profile, as defined in RFC 3551. The application
  *	can set any other profile instead using that function.
- *
+ * @param session a rtp session
+ * @param profile a rtp profile
  *
 **/
 
@@ -389,15 +396,12 @@ rtp_session_set_send_profile (RtpSession * session, RtpProfile * profile)
 
 
 /**
- *rtp_session_set_recv_profile:
- *@session: a rtp session
- *@profile: a rtp profile
- *
  *	Set the RTP profile to be used for the receiveing by this session. By default, all session are created by
  *	rtp_session_new() are initialized with the AV profile, as defined in RFC 3551. The application
  *	can set any other profile instead using that function.
  *
- *
+ * @param session a rtp session
+ * @param profile a rtp profile
 **/
 
 void
@@ -408,10 +412,10 @@ rtp_session_set_recv_profile (RtpSession * session, RtpProfile * profile)
 }
 
 /**
- *rtp_session_get_profile:
- *@session: a rtp session
+ *@param session a rtp session
  *
- *	Returns current profile.
+ *	DEPRECATED! Returns current send profile.
+ *	Use rtp_session_get_send_profile() or rtp_session_get_recv_profile()
  *
 **/
 RtpProfile *rtp_session_get_profile(RtpSession *session){
@@ -420,8 +424,7 @@ RtpProfile *rtp_session_get_profile(RtpSession *session){
 
 
 /**
- *rtp_session_get_send_profile:
- *@session: a rtp session
+ *@param session a rtp session
  *
  *	Returns current send profile.
  *
@@ -431,8 +434,7 @@ RtpProfile *rtp_session_get_send_profile(RtpSession *session){
 }
 
 /**
- *rtp_session_get_recv_profile:
- *@session: a rtp session
+ *@param session a rtp session
  *
  *	Returns current receive profile.
  *
@@ -442,26 +444,18 @@ RtpProfile *rtp_session_get_recv_profile(RtpSession *session){
 }
 
 /**
- *rtp_session_set_recv_buf_size:
- *@session: a rtp session
- *@bufsize: buffer size in bytes for receiving packets
- *
  *	The default value is 65535 bytes, a big value which is working for everyone.
  *	However if your application can make assumption on the MTU, it can be interesting
  *	to set it to a lower value in order to save memory.
  *
+ * @param session a rtp session
+ * @param bufsize max size in bytes for receiving packets
 **/
 void rtp_session_set_recv_buf_size(RtpSession *session, int bufsize){
 	session->recv_buf_size=bufsize;
 }
 
 /**
- *rtp_session_signal_connect:
- *@session: 	a rtp session
- *@signal_name:		the name of a signal
- *@cb:			a #RtpCallback
- *@user_data:	a pointer to any data to be passed when invoking the callback.
- *
  *	This function provides the way for an application to be informed of various events that
  *	may occur during a rtp session. @signal is a string identifying the event, and @cb is 
  *	a user supplied function in charge of processing it. The application can register
@@ -487,6 +481,12 @@ void rtp_session_set_recv_buf_size(RtpSession *session, int bufsize){
  * 
  *	Returns: 0 on success, -EOPNOTSUPP if the signal does not exists, -1 if no more callbacks
  *	can be assigned to the signal type.
+ *
+ * @param session 	a rtp session
+ * @param signal_name	the name of a signal
+ * @param cb		a RtpCallback
+ * @param user_data	a pointer to any data to be passed when invoking the callback.
+ *
 **/
 int
 rtp_session_signal_connect (RtpSession * session, const char *signal_name,
@@ -505,16 +505,13 @@ rtp_session_signal_connect (RtpSession * session, const char *signal_name,
 
 
 /**
- *rtp_session_signal_disconnect_by_callback:
- *@session: a rtp session
- *@signal_name:	a signal name
- *@cb:		a callback function.
- *
  *	Removes callback function @cb to the list of callbacks for signal @signal.
  *
- *Returns: 0 on success, -ENOENT if the callbacks was not found.
+ * @param session a rtp session
+ * @param signal_name	a signal name
+ * @param cb	a callback function.
+ * @return: 0 on success, a negative value if the callback was not found.
 **/
-
 int
 rtp_session_signal_disconnect_by_callback (RtpSession * session, const char *signal_name,
 					   RtpCallback cb)
@@ -532,11 +529,9 @@ rtp_session_signal_disconnect_by_callback (RtpSession * session, const char *sig
 
 
 /**
- *rtp_session_set_seq_number:
- *@session:		a rtp session freshly created.
- *@addr:			a 16 bit unsigned number.
- *
  * sets the initial sequence number of a sending session.
+ * @param session		a rtp session freshly created.
+ * @param addr			a 16 bit unsigned number.
  *
 **/
 void rtp_session_set_seq_number(RtpSession *session, uint16_t seq){
@@ -550,13 +545,11 @@ uint16_t rtp_session_get_seq_number(RtpSession *session){
 
 
 /**
- *rtp_session_set_ssrc:
- *@session: a rtp session.
- *@ssrc: an unsigned 32bit integer representing the synchronisation source identifier (SSRC).
- *
  *	Sets the SSRC for the outgoing stream.
  *  If not done, a random ssrc is used.
  *
+ * @param session a rtp session.
+ * @param ssrc an unsigned 32bit integer representing the synchronisation source identifier (SSRC).
 **/
 void
 rtp_session_set_ssrc (RtpSession * session, uint32_t ssrc)
@@ -578,17 +571,15 @@ void rtp_session_update_payload_type(RtpSession *session, int paytype){
 	}
 }
 /**
- *rtp_session_set_send_payload_type:
- *@session: a rtp session
- *@paytype: the payload type
- *
  *	Sets the payload type of the rtp session. It decides of the payload types written in the
  *	of the rtp header for the outgoing stream, if the session is SENDRECV or SENDONLY.
  *	For payload type in incoming packets, the application can be informed by registering
  *	for the "payload_type_changed" signal, so that it can make the necessary changes
  *	on the downstream decoder that deals with the payload of the packets.
  *
- *Returns: 0 on success, -1 if the payload is not defined.
+ * @param session a rtp session
+ * @param paytype the payload type number
+ * @return 0 on success, -1 if the payload is not defined.
 **/
 
 int
@@ -599,25 +590,23 @@ rtp_session_set_send_payload_type (RtpSession * session, int paytype)
 }
 
 /**
- *rtp_session_get_send_payload_type:
- *@session: a rtp session
+ *@param session a rtp session
  *
- *	Returns: the payload type currently used in outgoing rtp packets
+ *@return the payload type currently used in outgoing rtp packets
 **/
 int rtp_session_get_send_payload_type(const RtpSession *session){
 	return session->snd.pt;
 }
 
 /**
- *rtp_session_set_recv_payload_type:
- *@session: a rtp session
- *@paytype: the payload type
  *
  *	Sets the expected payload type for incoming packets.
  *	If the actual payload type in incoming packets is different that this expected payload type, thus
  *	the "payload_type_changed" signal is emitted.
  *
- *Returns: 0 on success, -1 if the payload is not defined.
+ *@param session a rtp session
+ *@param paytype the payload type number
+ *@return 0 on success, -1 if the payload is not defined.
 **/
 
 int
@@ -634,25 +623,22 @@ rtp_session_set_recv_payload_type (RtpSession * session, int paytype)
 }
 
 /**
- *rtp_session_get_recv_payload_type:
- *@session: a rtp session
+ *@param session a rtp session
  *
- *	Returns: the payload type currently used in incoming rtp packets
+ * @return the payload type currently used in incoming rtp packets
 **/
 int rtp_session_get_recv_payload_type(const RtpSession *session){
 	return session->rcv.pt;
 }
 
 /**
- *rtp_session_set_payload_type:
- *@session: a rtp session
- *@paytype: the payload type
- *
  *	Sets the expected payload type for incoming packets and payload type to be used for outgoing packets.
  *	If the actual payload type in incoming packets is different that this expected payload type, thus
  *	the "payload_type_changed" signal is emitted.
  *
- *Returns: 0 on success, -1 if the payload is not defined.
+ * @param session a rtp session
+ * @param paytype the payload type number
+ * @return 0 on success, -1 if the payload is not defined.
 **/
 int rtp_session_set_payload_type(RtpSession *session, int pt){
 	if (rtp_session_set_send_payload_type(session,pt)<0) return -1;
@@ -675,18 +661,16 @@ static void rtp_header_init_from_session(rtp_header_t *rtp, RtpSession *session)
 }
 
 /**
- *rtp_session_create_packet:
- *@session:		a rtp session.
- *@header_size:	the rtp header size. For standart size (without extensions), it is #RTP_FIXED_HEADER_SIZE
- *@payload		:data to be copied into the rtp packet.
- *@payload_size	: size of data carried by the rtp packet.
- *
  *	Allocates a new rtp packet. In the header, ssrc and payload_type according to the session's
  *	context. Timestamp and seq number are not set, there will be set when the packet is going to be
  *	sent with rtp_session_sendm_with_ts().
  *	If payload_size is zero, thus an empty packet (just a RTP header) is returned.
  *
- *Returns: a rtp packet in a mblk_t (message block) structure.
+ *@param session a rtp session.
+ *@param header_size the rtp header size. For standart size (without extensions), it is RTP_FIXED_HEADER_SIZE
+ *@param payload data to be copied into the rtp packet.
+ *@param payload_size size of data carried by the rtp packet.
+ *@return a rtp packet in a mblk_t (message block) structure.
 **/
 mblk_t * rtp_session_create_packet(RtpSession *session,int header_size, const uint8_t *payload, int payload_size)
 {
@@ -707,12 +691,6 @@ mblk_t * rtp_session_create_packet(RtpSession *session,int header_size, const ui
 }
 
 /**
- *rtp_session_create_packet_with_data:
- *@session:		a rtp session.
- *@payload		: the data to be sent with this packet
- *@payload_size	: size of data
- *@freefn		: a function that will be called when the payload buffer is no more needed.
- *
  *	Creates a new rtp packet using the given payload buffer (no copy). The header will be allocated separetely.
  *  In the header, ssrc and payload_type according to the session's
  *	context. Timestamp and seq number are not set, there will be set when the packet is going to be
@@ -721,7 +699,11 @@ mblk_t * rtp_session_create_packet(RtpSession *session,int header_size, const ui
  *	packet concatenation involving copies to be done in user-space.
  *  @freefn can be NULL, in that case payload will be kept untouched.
  *
- *Returns: a rtp packet in a mblk_t (message block) structure.
+ * @param session a rtp session.
+ * @param payload the data to be sent with this packet
+ * @param payload_size size of data
+ * @param freefn a function that will be called when the payload buffer is no more needed.
+ * @return: a rtp packet in a mblk_t (message block) structure.
 **/
 
 mblk_t * rtp_session_create_packet_with_data(RtpSession *session, uint8_t *payload, int payload_size, void (*freefn)(void*))
@@ -744,19 +726,17 @@ mblk_t * rtp_session_create_packet_with_data(RtpSession *session, uint8_t *paylo
 
 
 /**
- *rtp_session_create_packet_in_place:
- *@session:		a rtp session.
- *@buffer:	a buffer that contains first just enough place to write a RTP header, then the data to send.
- *@size		: the size of the buffer
- *@freefn : a function that will be called once the buffer is no more needed (the data has been sent).
- *
- *	Creates a new rtp packet using the buffer given in arguments (no copy). 
- *  In the header, ssrc and payload_type according to the session's
- *	context. Timestamp and seq number are not set, there will be set when the packet is going to be
+ * Creates a new rtp packet using the buffer given in arguments (no copy). 
+ * In the header, ssrc and payload_type according to the session's
+ *context. Timestamp and seq number are not set, there will be set when the packet is going to be
  *	sent with rtp_session_sendm_with_ts().
  *  @freefn can be NULL, in that case payload will be kept untouched.
  *
- *Returns: a rtp packet in a mblk_t (message block) structure.
+ * @param session a rtp session.
+ * @param buffer a buffer that contains first just enough place to write a RTP header, then the data to send.
+ * @param size the size of the buffer
+ * @param freefn a function that will be called once the buffer is no more needed (the data has been sent).
+ * @return a rtp packet in a mblk_t (message block) structure.
 **/
 mblk_t * rtp_session_create_packet_in_place(RtpSession *session,uint8_t *buffer, int size, void (*freefn)(void*) )
 {
@@ -771,17 +751,15 @@ mblk_t * rtp_session_create_packet_in_place(RtpSession *session,uint8_t *buffer,
 }
 
 /**
- *rtp_session_sendm_with_ts:
- *@session	: a rtp session.
- *@mp		:	a rtp packet presented as a mblk_t.
- *@timestamp:	the timestamp of the data to be sent. Refer to the rfc to know what it is.
- *
  *	Send the rtp datagram @mp to the destination set by rtp_session_set_remote_addr() 
  *	with timestamp @timestamp. For audio data, the timestamp is the number
  *	of the first sample resulting of the data transmitted. See rfc1889 for details.
  *  The packet (@mp) is freed once it is sended.
  *
- *Returns: the number of bytes sent over the network.
+ *@param session a rtp session.
+ *@param mp a rtp packet presented as a mblk_t.
+ *@param timestamp the timestamp of the data to be sent.
+ * @return the number of bytes sent over the network.
 **/
 int
 rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
@@ -854,25 +832,24 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 	error = rtp_session_rtp_send (session, mp);
 	/*send RTCP packet if needed */
 	rtp_session_rtcp_process_send(session);
-	/* receives rtcp packet */
-	rtp_session_rtcp_recv(session);
+	/* receives rtcp packet if session is send-only*/
+	/*otherwise it is done in rtp_session_recvm_with_ts */
+	if (session->mode==RTP_SESSION_SENDONLY) rtp_session_rtcp_recv(session);
 	return error;
 }
 
 
 /**
- *rtp_session_send_with_ts:
- *@session: a rtp session.
- *@buffer:	a buffer containing the data to be sent in a rtp packet.
- *@len:		the length of the data buffer, in bytes.
- *@userts:	the timestamp of the data to be sent. Refer to the rfc to know what it is.
- *
  *	Send a rtp datagram to the destination set by rtp_session_set_remote_addr() containing
  *	the data from @buffer with timestamp @userts. This is a high level function that uses
  *	rtp_session_create_packet() and rtp_session_sendm_with_ts() to send the data.
  *
+ *@param session a rtp session.
+ *@param buffer a buffer containing the data to be sent in a rtp packet.
+ *@param len the length of the data buffer, in bytes.
+ *@param userts	the timestamp of the data to be sent. Refer to the rfc to know what it is.
  *
- *Returns: the number of bytes sent over the network.
+ *@param return the number of bytes sent over the network.
 **/
 int
 rtp_session_send_with_ts (RtpSession * session, const uint8_t * buffer, int len,
@@ -902,10 +879,6 @@ static void payload_type_changed_notify(RtpSession *session, int paytype){
 
 
 /**
- *rtp_session_recvm_with_ts:
- *@session: a rtp session.
- *@user_ts:	a timestamp.
- *
  *	Try to get a rtp packet presented as a mblk_t structure from the rtp session.
  *	The @user_ts parameter is relative to the first timestamp of the incoming stream. In other
  *	words, the application does not have to know the first timestamp of the stream, it can
@@ -913,7 +886,10 @@ static void payload_type_changed_notify(RtpSession *session, int paytype){
  *	as it want. The RtpSession takes care of synchronisation between the stream timestamp
  *	and the user timestamp given here.
  *
- *Returns: a rtp packet presented as a mblk_t.
+ * @param session a rtp session.
+ * @param user_ts a timestamp.
+ *
+ * @return a rtp packet presented as a mblk_t.
 **/
 
 mblk_t *
@@ -926,6 +902,7 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 	RtpScheduler *sched=session->sched;
 	RtpStream *stream=&session->rtp;
 	int rejected=0;
+	bool_t read_socket=TRUE;
 
 	/* if we are scheduled, remember the scheduler time at which the application has
 	 * asked for its first timestamp */
@@ -944,10 +921,17 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 			//ortp_message("setting snd_time_offset=%i",session->rtp.snd_time_offset);
 		}
 		rtp_session_unset_flag (session,RTP_SESSION_RECV_NOT_STARTED);
+	}else{
+		/*prevent reading from the sockets when two 
+		consecutives calls for a same timestamp*/
+		if (user_ts==session->rtp.rcv_last_app_ts)
+			read_socket=FALSE;
 	}
 	session->rtp.rcv_last_app_ts = user_ts;
-	rtp_session_rtp_recv (session, user_ts);
-	rtp_session_rtcp_recv(session);
+	if (read_socket){
+		rtp_session_rtp_recv (session, user_ts);
+		rtp_session_rtcp_recv(session);
+	}
 	/* check for telephone event first */
 	mp=getq(&session->rtp.tev_rq);
 	if (mp!=NULL){
@@ -986,7 +970,7 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 	}
 	/*calculate the stream timestamp from the user timestamp */
 	ts = user_ts + session->rtp.rcv_diff_ts;
-	if (session->permissive || session->rtp.jittctl.jitt_comp_ts==0)
+	if (session->permissive || session->rtp.jittctl.enabled==FALSE)
 		mp = rtp_getq_permissive(&session->rtp.rq, ts,&rejected);
 	else
 		mp = rtp_getq(&session->rtp.rq, ts,&rejected);
@@ -1094,16 +1078,13 @@ static int msg_to_buf (mblk_t * mp, uint8_t *buffer, int len)
 }
 
 /**
- *rtp_session_recv_with_ts:
- *@session: a rtp session.
- *@buffer:	a user supplied buffer to write the data.
- *@len:		the length in bytes of the user supplied buffer.
- *@ts:		the timestamp wanted.
- *@have_more: the address of an integer to indicate if more data is availlable for the given timestamp.
+ *	NOTE: use of this function is discouraged when sending payloads other than
+ *	pcm/pcmu/pcma/adpcm types.
+ *	rtp_session_recvm_with_ts() does better job.
  *
- *	Tries to read the bytes of the incoming rtp stream related to timestamp @ts. In case 
+ *	Tries to read the bytes of the incoming rtp stream related to timestamp ts. In case 
  *	where the user supplied buffer @buffer is not large enough to get all the data 
- *	related to timestamp @ts, then *( @have_more) is set to 1 to indicate that the application
+ *	related to timestamp ts, then *( have_more) is set to 1 to indicate that the application
  *	should recall the function with the same timestamp to get more data.
  *	
  *  When the rtp session is scheduled (see rtp_session_set_scheduling_mode() ), and the 
@@ -1128,6 +1109,12 @@ static int msg_to_buf (mblk_t * mp, uint8_t *buffer, int len)
  *	are availlable, either because the sender has not started to send the stream, or either
  *	because silence packet are not transmitted, or either because the packet was lost during
  *	network transport, then the function returns zero.
+ *@param session a rtp session.
+ *@param buffer a user supplied buffer to write the data.
+ *@param len the length in bytes of the user supplied buffer.
+ *@param ts the timestamp wanted.
+ *@param have_more the address of an integer to indicate if more data is availlable for the given timestamp.
+ *
 **/
 int rtp_session_recv_with_ts (RtpSession * session, uint8_t * buffer,
 			       int len, uint32_t ts, int * have_more)
@@ -1251,9 +1238,6 @@ int rtp_session_recv_with_ts (RtpSession * session, uint8_t * buffer,
 	return -1;
 }
 /**
- *rtp_session_get_current_send_ts:
- *@session: a rtp session.
- *
  *	When the rtp session is scheduled and has started to send packets, this function
  *	computes the timestamp that matches to the present time. Using this function can be 
  *	usefull when sending discontinuous streams. Some time can be elapsed between the end
@@ -1262,7 +1246,8 @@ int rtp_session_recv_with_ts (RtpSession * session, uint8_t * buffer,
  *	#rtp_session_send_with_ts() or #rtp_session_sendm_with_ts(), the application may
  *	use rtp_session_get_current_send_ts().
  *
- *Returns: the current send timestamp for the rtp session.
+ * @param session a rtp session.
+ * @return the current send timestamp for the rtp session.
 **/
 uint32_t rtp_session_get_current_send_ts(RtpSession *session)
 {
@@ -1283,13 +1268,11 @@ uint32_t rtp_session_get_current_send_ts(RtpSession *session)
 }
 
 /**
- *rtp_session_get_current_recv_ts:
- *@session: a rtp session.
- *
  * Same thing as rtp_session_get_current_send_ts() except that it's for an incoming stream.
  * Works only on scheduled mode.
  *
- * Returns: the theoritical that would have to be receive now.
+ * @param session a rtp session.
+ * @return the theoritical that would have to be receive now.
  *
 **/
 uint32_t rtp_session_get_current_recv_ts(RtpSession *session){
@@ -1310,15 +1293,14 @@ uint32_t rtp_session_get_current_recv_ts(RtpSession *session){
 }
 
 /**
- *rtp_session_set_time_jump_limit:
- *@session: the rtp session
- *@ts_step: a time interval in miliseconds
- *
  * oRTP has the possibility to inform the application through a callback registered 
  * with rtp_session_signal_connect about crazy incoming RTP stream that jumps from 
  * a timestamp N to N+some_crazy_value. This lets the opportunity for the application
  * to reset the session in order to resynchronize, or any other action like stopping the call
  * and reporting an error.
+ * @param session the rtp session
+ * @param ts_step a time interval in miliseconds
+ *
 **/
 void rtp_session_set_time_jump_limit(RtpSession *session, int milisecs){
 	uint32_t ts;
@@ -1328,6 +1310,9 @@ void rtp_session_set_time_jump_limit(RtpSession *session, int milisecs){
 	else session->rtp.ts_jump=ts;
 }
 
+/**
+ * Closes the rtp and rtcp sockets.
+**/
 void rtp_session_release_sockets(RtpSession *session){
 	if (session->rtp.socket>=0) close_socket (session->rtp.socket);
 	if (session->rtcp.socket>=0) close_socket (session->rtcp.socket);
@@ -1350,6 +1335,10 @@ ortp_socket_t rtp_session_get_rtcp_socket(const RtpSession *session){
 	return rtp_session_using_transport(session, rtcp) ? (session->rtcp.tr->t_getsocket)(session->rtcp.tr) : session->rtcp.socket;
 }
 
+/**
+ * Register an event queue.
+ * An application can use an event queue to get informed about various RTP events.
+**/
 void rtp_session_register_event_queue(RtpSession *session, OrtpEvQueue *q){
 	session->eventqs=o_list_append(session->eventqs,q);
 }
@@ -1359,27 +1348,12 @@ void rtp_session_unregister_event_queue(RtpSession *session, OrtpEvQueue *q){
 }
 
 void rtp_session_dispatch_event(RtpSession *session, OrtpEvent *ev){
-#if 0
-	if (session->eventqs==NULL) ortp_event_destroy(ev);
-	else{
-		OList *it;
-		int i;
-		for(i=0,it=session->eventqs;it!=NULL;it=it->next,++i){
-			if (i==0) ortp_ev_queue_put((OrtpEvQueue*)it->data,ev);
-			else  ortp_ev_queue_put((OrtpEvQueue*)it->data,ortp_event_dup(ev));
-		}
-	}
-#else
-	{
-	  OList *it;
-	  int i;
-	  for(i=0,it=session->eventqs;it!=NULL;it=it->next,++i)
-	    {
-	      ortp_ev_queue_put((OrtpEvQueue*)it->data,ortp_event_dup(ev));
-	    }
+	OList *it;
+	int i;
+	for(i=0,it=session->eventqs;it!=NULL;it=it->next,++i){
+		ortp_ev_queue_put((OrtpEvQueue*)it->data,ortp_event_dup(ev));
 	}	
 	ortp_event_destroy(ev);
-#endif
 }
 
 
@@ -1409,12 +1383,10 @@ void rtp_session_uninit (RtpSession * session)
 }
 
 /**
- * rtp_session_resync:
- * @session: the rtp session
- *
  * Resynchronize to the incoming RTP streams.
  * This can be useful to handle discoutinuous timestamps.
  * For example, call this function from the timestamp_jump signal handler.
+ * @param session the rtp session
 **/
 void rtp_session_resync(RtpSession *session){
 	flushq (&session->rtp.rq, FLUSHALL);
@@ -1424,13 +1396,10 @@ void rtp_session_resync(RtpSession *session){
 }
 
 /**
- *rtp_session_reset:
- *@session: a rtp session.
- *
- *	Reset the session: local and remote addresses are kept unchanged but the internal
- *	queue for ordering and buffering packets is flushed, the session is ready to be
- *	re-synchronised to another incoming stream.
+ * Reset the session: local and remote addresses are kept. It resets timestamp, sequence 
+ * number, and calls rtp_session_resync().
  *
+ * @param session a rtp session.
 **/
 void rtp_session_reset (RtpSession * session)
 {
@@ -1454,6 +1423,9 @@ void rtp_session_reset (RtpSession * session)
 	rtp_session_resync(session);
 }
 
+/**
+ * Retrieve the session's statistics.
+**/
 const rtp_stats_t * rtp_session_get_stats(const RtpSession *session){
 	return &session->rtp.stats;
 }
@@ -1463,12 +1435,9 @@ void rtp_session_reset_stats(RtpSession *session){
 }
 
 /**
- *rtp_session_set_data:
- *	@session : a rtp session
- *	@data : an opaque pointer to be stored in the session
- *
- *	Stores some application specific data into the session, so that it is easy to retrieve it
- *	from the signal callbacks using rtp_session_get_data().
+ * Stores some application specific data into the session, so that it is easy to retrieve it from the signal callbacks using rtp_session_get_data().
+ * @param session a rtp session
+ * @param data an opaque pointer to be stored in the session
 **/
 
 void rtp_session_set_data(RtpSession *session, void *data){
@@ -1476,23 +1445,21 @@ void rtp_session_set_data(RtpSession *session, void *data){
 }
 
 /**
- *rtp_session_get_data:
- *	@session : a rtp session
- *
- *	Returns: the void pointer previously set using rtp_session_set_data()
+ * @param session a rtp session
+ * @return the void pointer previously set using rtp_session_set_data()
 **/
 void *rtp_session_get_data(const RtpSession *session){
 	return session->user_data;
 }
 
 /**
- *rtp_session_set_symmetric_rtp:
- *	@session : a rtp session
- *	@yesno : a boolean to enable or disable the feature
+ * Enable or disable the "rtp symmetric" hack which consists of the following:
+ * after the first packet is received, the source address of the packet 
+ * is set to be the destination address for all next packets.
+ * This is useful to pass-through firewalls.
+ * @param session a rtp session
+ * @param yesno a boolean to enable or disable the feature
  *
- *	Enable or disable the "rtp symmetric" hack: after the first packet is
- *	received, the source address of the packet is set to be the destination
- *	address for all nexts packets.
 **/
 void
 rtp_session_set_symmetric_rtp (RtpSession * session, bool_t yesno)
@@ -1501,10 +1468,6 @@ rtp_session_set_symmetric_rtp (RtpSession * session, bool_t yesno)
 }
 
 /**
- *rtp_session_set_connected_mode:
- *	@session : a rtp session
- *	@yesno : a boolean to enable or disable the feature
- *
  *	If yesno is TRUE, thus a connect() syscall is done on the socket to 
  *	the destination address set by rtp_session_set_remote_addr(), or
  *	if the session does symmetric rtp (see rtp_session_set_symmetric_rtp())
@@ -1513,6 +1476,9 @@ rtp_session_set_symmetric_rtp (RtpSession * session, bool_t yesno)
  *	don't come from the address specified in connect().
  *	It also makes ICMP errors (such as connection refused) available to the
  *	application.
+ *	@param session a rtp session
+ *	@param yesno a boolean to enable or disable the feature
+ *
 **/
 void rtp_session_set_connected_mode(RtpSession *session, bool_t yesno){
 	session->use_connect=yesno;
@@ -1524,9 +1490,10 @@ static float compute_bw(struct timeval *orig, unsigned int bytes){
 	float time;
 	if (bytes==0) return 0;
 	gettimeofday(&current,NULL);
-	time=(current.tv_sec - orig->tv_sec) +
-		((current.tv_usec - orig->tv_usec)*1e-6);
-	bw=((float)bytes)*8/time;
+	time=(float)(current.tv_sec - orig->tv_sec) +
+		((float)(current.tv_usec - orig->tv_usec)*1e-6);
+	bw=((float)bytes)*8/(time+0.001); 
+	/*+0.0001 avoids a division by zero without changing the results significatively*/
 	return bw;
 }
 
@@ -1546,11 +1513,10 @@ float rtp_session_compute_send_bandwidth(RtpSession *session){
 
 
 /**
- *rtp_session_destroy:
- *@session: a rtp session.
- *
- *	Destroys a rtp session.
+ * Destroys a rtp session.
+ * All memory allocated for the RtpSession is freed.
  *
+ * @param session a rtp session.
 **/
 void rtp_session_destroy (RtpSession * session)
 {
@@ -1558,8 +1524,6 @@ void rtp_session_destroy (RtpSession * session)
 	ortp_free (session);
 }
 
-
-
 void rtp_session_make_time_distorsion(RtpSession *session, int milisec)
 {
 	session->rtp.snd_time_offset+=milisec;
@@ -1577,16 +1541,19 @@ void rtp_add_csrc(mblk_t *mp, uint32_t csrc)
 
 
 /**
- *rtp_session_get_last_recv_time:
- *@session: #RtpSession to get last receive time from.
- *@tv:      Pointer to struct timeval to fill.
- *
  *  Gets last time a valid RTP or RTCP packet was received.
+ * @param session RtpSession to get last receive time from.
+ * @param tv Pointer to struct timeval to fill.
+ *
 **/
 void
 rtp_session_get_last_recv_time(RtpSession *session, struct timeval *tv)
 {
-    *tv = session->last_recv_time;
+#ifdef PERF
+	ortp_error("rtp_session_get_last_recv_time() feature disabled.");
+#else
+    	*tv = session->last_recv_time;
+#endif
 }
 
 
