@@ -633,6 +633,11 @@ void rtp_session_set_transports(RtpSession *session, struct _RtpTransport *rtptr
 {
 	session->rtp.tr = rtptr;
 	session->rtcp.tr = rtcptr;
+	if (rtptr)
+		rtptr->session=session;
+	if (rtcptr)
+		rtcptr->session=session;
+
 	if (rtptr || rtcptr )
 		session->flags|=(RTP_SESSION_USING_TRANSPORT);
 	else session->flags&=~(RTP_SESSION_USING_TRANSPORT);
@@ -651,7 +656,7 @@ void rtp_session_set_transports(RtpSession *session, struct _RtpTransport *rtptr
  *
 **/
 void rtp_session_flush_sockets(RtpSession *session){
-	char trash[4096];
+	unsigned char trash[4096];
 #ifdef ORTP_INET6
 	struct sockaddr_storage from;
 #else
@@ -660,11 +665,13 @@ void rtp_session_flush_sockets(RtpSession *session){
 	socklen_t fromlen=sizeof(from);
 	if (rtp_session_using_transport(session, rtp))
 	  {
-	    while (session->rtp.tr->t_recvfrom(session->rtp.tr,trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
+		mblk_t *trashmp=esballoc(trash,sizeof(trash),0,NULL);
+		
+	    while (session->rtp.tr->t_recvfrom(session->rtp.tr,trashmp,0,(struct sockaddr *)&from,&fromlen)>0){};
 
 	    if (session->rtcp.tr)
-	      while (session->rtcp.tr->t_recvfrom(session->rtcp.tr,trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
-	    
+	      while (session->rtcp.tr->t_recvfrom(session->rtcp.tr,trashmp,0,(struct sockaddr *)&from,&fromlen)>0){};
+		freemsg(trashmp);
 	    return;
 	  }
 
@@ -688,6 +695,9 @@ static int rtp_sendmsg(int sock,mblk_t *m, struct sockaddr *rem_addr, int addr_l
 		iov[iovlen].iov_base=m->b_rptr;
 		iov[iovlen].iov_len=m->b_wptr-m->b_rptr;
 	}
+	if (iovlen==MAX_IOV){
+		ortp_error("Too long msgb, didn't fit into iov, end discarded.");
+	}
 	msg.msg_name=(void*)rem_addr;
 	msg.msg_namelen=addr_len;
 	msg.msg_iov=&iov[0];
@@ -738,20 +748,19 @@ rtp_session_rtp_send (RtpSession * session, mblk_t * m)
 		destaddr=NULL;
 		destlen=0;
 	}
-#ifdef USE_SENDMSG
-	error=rtp_sendmsg(sockfd,m,destaddr,destlen);
-#else
-	if (m->b_cont!=NULL){
-		msgpullup(m,-1);
-	}
 
-	if (rtp_session_using_transport(session, rtp))
-	  error = (session->rtp.tr->t_sendto) (session->rtp.tr, m->b_rptr, (int) (m->b_wptr - m->b_rptr),
-			 0,destaddr,destlen);
-	else
-	  error = sendto (sockfd, m->b_rptr, (int) (m->b_wptr - m->b_rptr),
+	if (rtp_session_using_transport(session, rtp)){
+		error = (session->rtp.tr->t_sendto) (session->rtp.tr,m,0,destaddr,destlen);
+	}else{
+#ifdef USE_SENDMSG
+		error=rtp_sendmsg(sockfd,m,destaddr,destlen);
+#else
+		if (m->b_cont!=NULL)
+			msgpullup(m,-1);
+		error = sendto (sockfd, m->b_rptr, (int) (m->b_wptr - m->b_rptr),
 			 0,destaddr,destlen);
 #endif
+	}
 	if (error < 0){
 		if (session->on_network_error.count>0){
 			rtp_signal_table_emit3(&session->on_network_error,(long)"Error sending RTP packet",INT_TO_POINTER(getSocketErrorCode()));
@@ -777,30 +786,32 @@ rtp_session_rtcp_send (RtpSession * session, mblk_t * m)
 		destlen=0;
 	}
 
-	if ((sockfd>=0 || rtp_session_using_transport(session, rtcp)) && (session->rtcp.rem_addrlen>0 ||using_connected_socket) ){
-		
-#ifndef USE_SENDMSG
-		if (m->b_cont!=NULL){
-			msgpullup(m,-1);
+	if (session->rtcp.enabled &&
+		( (sockfd>=0 && (session->rtcp.rem_addrlen>0 ||using_connected_socket))
+			|| rtp_session_using_transport(session, rtcp) ) ){
+		if (rtp_session_using_transport(session, rtcp)){
+			error = (session->rtcp.tr->t_sendto) (session->rtcp.tr, m, 0,
+			destaddr, destlen);
 		}
-		if (rtp_session_using_transport(session, rtcp))
-		  error = (session->rtcp.tr->t_sendto) (sockfd, m->b_rptr,
-			(int) (m->b_wptr - m->b_rptr), 0,
-			destaddr, destlen);
-		else
-		  error = sendto (sockfd, m->b_rptr,
-			(int) (m->b_wptr - m->b_rptr), 0,
-			destaddr, destlen);
+		else{
+#ifdef USE_SENDMSG
+			error=rtp_sendmsg(sockfd,m,destaddr, destlen);
 #else
-		error=rtp_sendmsg(sockfd,m,destaddr, destlen);
+			if (m->b_cont!=NULL){
+				msgpullup(m,-1);
+			}
+			error = sendto (sockfd, m->b_rptr,
+			(int) (m->b_wptr - m->b_rptr), 0,
+			destaddr, destlen);
 #endif
+		}
 		if (error < 0){
-			char host[50];
+			char host[65];
 			if (session->on_network_error.count>0){
 				rtp_signal_table_emit3(&session->on_network_error,(long)"Error sending RTCP packet",INT_TO_POINTER(getSocketErrorCode()));
 			}else ortp_warning ("Error sending rtcp packet: %s ; socket=%i; addr=%s", getSocketError(), session->rtcp.socket, ortp_inet_ntoa((struct sockaddr*)&session->rtcp.rem_addr,session->rtcp.rem_addrlen,host,sizeof(host)) );
 		}
-	}else ortp_warning("Cannot send rtcp report: sockfd=%i, rem_addrlen=%i, connected=%i",sockfd,session->rtcp.rem_addrlen,using_connected_socket);
+	}else ortp_message("Not sending rtcp report: sockfd=%i, rem_addrlen=%i, connected=%i",sockfd,session->rtcp.rem_addrlen,using_connected_socket);
 	freemsg (m);
 	return error;
 }
@@ -817,13 +828,14 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 #endif
 	socklen_t addrlen = sizeof (remaddr);
 	mblk_t *mp;
-	bool_t sock_connected=(session->flags & RTP_SOCKET_CONNECTED);
-
+	
 	if ((sockfd<0) && !rtp_session_using_transport(session, rtp)) return -1;  /*session has no sockets for the moment*/
 
 	while (1)
 	{
 		int bufsz;
+		bool_t sock_connected=!!(session->flags & RTP_SOCKET_CONNECTED);
+
 		if (session->rtp.cached_mp==NULL)
 			 session->rtp.cached_mp = allocb (session->recv_buf_size, 0);
 		mp=session->rtp.cached_mp;
@@ -831,8 +843,7 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 		if (sock_connected){
 			error=recv(sockfd,mp->b_wptr,bufsz,0);
 		}else if (rtp_session_using_transport(session, rtp)) 
-		  error = (session->rtp.tr->t_recvfrom)(session->rtp.tr, mp->b_wptr,
-				  bufsz, 0,
+			error = (session->rtp.tr->t_recvfrom)(session->rtp.tr, mp, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
 		else error = recvfrom(sockfd, mp->b_wptr,
@@ -900,13 +911,13 @@ rtp_session_rtcp_recv (RtpSession * session)
 #endif
 	socklen_t addrlen=0;
 	mblk_t *mp;
-	bool_t sock_connected=session->flags & RTCP_SOCKET_CONNECTED;
 
 	if (session->rtcp.socket<0 && !rtp_session_using_transport(session, rtcp)) return -1;  /*session has no rtcp sockets for the moment*/
 	
 
 	while (1)
 	{
+		bool_t sock_connected=!!(session->flags & RTCP_SOCKET_CONNECTED);
 		if (session->rtcp.cached_mp==NULL)
 			 session->rtcp.cached_mp = allocb (RTCP_MAX_RECV_BUFSIZE, 0);
 		
@@ -917,8 +928,7 @@ rtp_session_rtcp_recv (RtpSession * session)
 			addrlen=sizeof (remaddr);
 
 			if (rtp_session_using_transport(session, rtcp))
-			  error=(session->rtcp.tr->t_recvfrom)(session->rtcp.tr, mp->b_wptr,
-				  RTCP_MAX_RECV_BUFSIZE, 0,
+			  error=(session->rtcp.tr->t_recvfrom)(session->rtcp.tr, mp, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
 			else
