@@ -226,16 +226,13 @@ void msgpullup(mblk_t *mp,int len)
 {
 	mblk_t *firstm=mp;
 	dblk_t *db;
-	int msgsize;
 	int wlen=0;
-	
-	if (mp->b_cont==NULL) return;	/*nothing to do, message is not fragmented */
-	
-	msgsize=msgdsize(mp);
-	if ((len==-1) || (len>msgsize)) len=msgsize;
-	db=datab_alloc(len);
 
-	while(wlen<len){
+	if (mp->b_cont==NULL && len==-1) return;	/*nothing to do, message is not fragmented */
+
+	if (len==-1) len=msgdsize(mp);
+	db=datab_alloc(len);
+	while(wlen<len && mp!=NULL){
 		int remain=len-wlen;
 		int mlen=mp->b_wptr-mp->b_rptr;
 		if (mlen<=remain){
@@ -253,7 +250,7 @@ void msgpullup(mblk_t *mp,int len)
 	datab_unref(firstm->b_datap);
 	firstm->b_datap=db;
 	firstm->b_rptr=db->db_base;
-	firstm->b_wptr=db->db_lim;
+	firstm->b_wptr=firstm->b_rptr+wlen;
 }
 
 
